/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.adapter;

import org.jabotics.robot.core.CSessionManager;
import org.jabotics.robot.defs.*;
import org.jabotics.robot.nxt.NXTAdapter;

/**
 * Provides the adapter factory for the NXT implementation.
 * 
 * @author  Frank Gollnik
 * @version 1.2.1
 */
public class XAdapter
{
    static private volatile IAdapterFactory sFactory = null;

    // The initialization of the adapter factory has to be triggered explicitly
    // before accessing the adapter factory from the core implementation.
    // Note: The core implementation will access the adapter factory during
    // CSessionManager.initSession() and CSessionManager.startSession().
    static public synchronized void initFactory()
    {
    	if (null == sFactory)
    	{
    		sFactory = NXTAdapter.getAdapterFactory();
    		// After sFactory has been set, the core session can be initialized and started:
    		CSessionManager.initSession();
    		CSessionManager.startSession();
    	}
    }

    static public IAdapterFactory getFactory()
    {
    	if (null == sFactory) throw new IllegalStateException("XAdapter.getFactory(): Method must not be called before adapter factory has been explicitly initilized by method initFactory().");

    	return sFactory;
    }
}
