/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.nxt;

import org.jabotics.robot.defs.*;

/**
 * NXTAdapter provides a handle to the interface IAdapterFactory for the NXT robot
 * and provides the implementation for this interface.
 * 
 * @author  Frank Gollnik
 * @version 1.2.1
 */
public class NXTAdapter
{
    static private final IAdapterFactory sFactory = new Factory();

    static public IAdapterFactory getAdapterFactory()
    {
        return sFactory;
    }

    static class Factory implements IAdapterFactory
    {
        private Factory()
        {
        }
    
        @Override
        public IRobotAdapter getRobotAdapter()
        {
            return ARobot.getAdapter();
        }

        @Override
        public IMotorAdapter getMotorAdapter(EMotorPort port)
        {
            return AMotor.getMotor(port);
        }
    
        @Override
        public ISensorAdapter getSensorAdapter(ESensorPort port,ESensorType type)
        {
            return ASensor.getSensorAdapter(port,type);
        }
    
        @Override
        public ISensorAdapter getButtonAdapter(EButtonType type)
        {
            return ASensor.getButtonAdapter(type);
        }
    
        @Override
        public ISensorAdapter getInvalidSensorAdapter()
        {
            return ASensor.getInvalidAdapter();
        }
    
        @Override
        public ISoundAdapter getSoundAdapter()
        {
            return ASound.getAdapter();
        }
    
        @Override
        public IDisplayAdapter getDisplayAdapter()
        {
            return ADisplay.getAdapter();
        }
    
        @Override
        public ITimerAdapter getTimerAdapter()
        {
            return ATimer.getAdapter();
        }
    
        @Override
        public IConsoleAdapter getConsoleAdapter()
        {
            return AConsole.getAdapter();
        }

		@Override
		public ISessionAdapter getSessionAdapter()
		{
			return null;
		}
    }
}
