/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.en;

/**
 * <span class="expert">&bull;</span> Objects supporting the interface ISensorListener
 * can be registered at a sensor to receive change event notifications of the sensor.
 * 
 * @author  Frank Gollnik
 * @version 1.2.1
 * @see     ISensor#setListener(ISensorListener)
 * @see     ISensor#clearListener()
 */

public interface ISensorListener
{
    /**
     * <span class="expert">&bull;</span> This method will be invoked when there is a relevant change
     * of the measurand or the corresponding change rate of an observed sensor.
     * A sensor listener needs to be registered at a sensor to receive change event
     * notifications (see {@link ISensor#setListener(ISensorListener)}).
     * The settings of the event filter tolerances will determine if a change of
     * the measurand or its change rate is being considered as relevant or not
     * (see {@link ISensor#setFilterTolerances(float,float)}).
     * Change events will be handled within a special event handling thread.
     * Special care is necessary in this situation to avoid severe problems at runtime:
     * The execution of the method manageEvent needs to be VERY FAST.
     * The real handling of the event should be done within the main thread of the robot program.
     * The method manageEvent should simply collect all data necessary
     * for the main thread to react to sensor events in an appropriate manner.
     * In particular, do not make calls to methods manipulating motors or sensors from the method manageEvent.
     * It will not only take too much time but will easily mess with the main threads business.
     * Reading of values from motors and sensors will in general be OK.
     * In turn, the main thread should not manipulate data that the method manageEvent
     * is responsible for when monitoring sensors.
     * 
     * @param   sensor     calling sensor
     * @param   oldValue   old measurand
     * @param   newValue   new measurand
     * @param   oldRate    old change rate
     * @param   newRate    new change rate
     * @see     ISensor#setListener(ISensorListener)
     * @see     ISensor#clearListener()
     * @see     ISensor#setFilterTolerances(float,float)
     * @see     ISensor#getFilterToleranceValue()
     * @see     ISensor#getFilterToleranceRate()
     * @see     ISensor#checkForEventOverflow()
     */
    void manageEvent(ISensor sensor, float oldValue, float newValue, float oldRate, float newRate);
}
