package org.jabotics.robot.sim;

public abstract class BSessionStateListener implements ISessionHandler
{
	private XSession mSession = null;
	private ESessionState mSessionState = null;
	

	public XSession getSession()
	{
		return mSession;
	}
	
	public ESessionState getSessionState()
	{
		return mSessionState;
	}
	
	@Override
	public void handlerAdded(XSession session)
	{
		mSession = session;
		mSessionState = session.getState();
	}

	@Override
	public void handlerRemoved(XSession session)
	{
		mSession = null;
	}

	@Override
	public void initSession()
	{
		sessionStateChanging(mSessionState, ESessionState.INITIALIZED);
		mSessionState = ESessionState.INITIALIZED;
	}

	@Override
	public void startSession() {}
	{
		sessionStateChanging(mSessionState, ESessionState.RUNNING);
		mSessionState = ESessionState.RUNNING;
	}

	@Override
	public void pauseSession() {}
	{
		sessionStateChanging(mSessionState, ESessionState.PAUSING);
		mSessionState = ESessionState.PAUSING;
	}

	@Override
	public void resumeSession() {}
	{
		sessionStateChanging(mSessionState, ESessionState.RUNNING);
		mSessionState = ESessionState.RUNNING;
	}

	@Override
	public void stopSession() {}
	{
		sessionStateChanging(mSessionState, ESessionState.STOPPED);
		mSessionState = ESessionState.STOPPED;
	}

	@Override
	public ISessionHandler completeSession()
	{
		sessionStateChanging(mSessionState, ESessionState.COMPLETED);
		mSessionState = ESessionState.COMPLETED;
		return this;
	}

	@Override
	public void terminateSession()
	{
		sessionStateChanging(mSessionState, ESessionState.TERMINATING);
		mSessionState = ESessionState.TERMINATING;
	}

    abstract public void sessionStateChanging(ESessionState oldState, ESessionState newState);
}
