/********************************************************************************************
*
* Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
*
* 
* This file is part of the Java software package org.jabotics (in the following: Jabotics).
* 
* Jabotics is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* Jabotics is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
* 
* 
* Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
* 
* Jabotics ist Freie Software: Sie können es unter den Bedingungen
* der GNU General Public License, wie von der Free Software Foundation,
* Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
* veröffentlichten Version, weiterverbreiten und/oder modifizieren.
* 
* Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
* OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
* Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
* Siehe die GNU General Public License für weitere Details.
* 
* Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
* erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
* 
*******************************************************************************************/


package org.jabotics.robot.sim;

import java.io.File;
import java.util.UUID;

import org.jabotics.pc.defs.SIMDefs;

/**
 * @author  Frank Gollnik
 * @version 1.2.1
 */
public class Env
{
    // constants:
    static public final boolean WINDOWS = ('\\' == File.separatorChar);
    static public final EContext CONTEXT = EnvInit.CONTEXT;
    static public final boolean SIM_UUID_IS_PREDEFINED = (null != System.getProperty(SIMDefs.KEY_SIM_UUID));
    static public final String SIM_UUID = SIM_UUID_IS_PREDEFINED ? System.getProperty(SIMDefs.KEY_SIM_UUID) : UUID.randomUUID().toString();
    static public final boolean ROBOT_IS_MASTER = (CONTEXT.equals(EContext.ROBOT)) ? !SIM_UUID_IS_PREDEFINED : SIM_UUID_IS_PREDEFINED;
    static public final int RMI_PORT = getRMIPort();
    // For the moment, disable the possibility to run robot as master in separate process:
    static public final boolean ROBOT_RUNS_IN_SEPARATE_PROCESS = (RMI_PORT > 0) && !ROBOT_IS_MASTER;
    //static public final boolean ROBOT_RUNS_IN_SEPARATE_PROCESS = (RMI_PORT > 0);
    static public final String RMI_SERVER = "//localhost:" + RMI_PORT + "/" + SIMDefs.KEY_RMI_SIMULATION;
    static public final String CLASS_PATH = System.getProperty("java.class.path");

    static private int getRMIPort()
    {
        String port = System.getProperty(SIMDefs.KEY_RMI_PORT);
        if (null == port) port = System.getenv(SIMDefs.KEY_RMI_PORT_ENV);
        int n_port = -1;
        if (null != port)
        {
            try {n_port = Integer.parseInt(port);} catch (Exception e) {}
            if (0 == n_port) n_port = 1099; // default port for RMI
        }
        return n_port;
    }

    static public void init() {}
}
