/********************************************************************************************
*
* Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
*
* 
* This file is part of the Java software package org.jabotics (in the following: Jabotics).
* 
* Jabotics is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* Jabotics is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
* 
* 
* Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
* 
* Jabotics ist Freie Software: Sie können es unter den Bedingungen
* der GNU General Public License, wie von der Free Software Foundation,
* Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
* veröffentlichten Version, weiterverbreiten und/oder modifizieren.
* 
* Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
* OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
* Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
* Siehe die GNU General Public License für weitere Details.
* 
* Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
* erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
* 
*******************************************************************************************/


package org.jabotics.robot.sim;

import java.rmi.Remote;
import java.rmi.RemoteException;

/**
 * @author  Frank Gollnik
 * @version 1.2.1
 */
interface ISimulationServer extends Remote
{
    /**************************************************************
     * IMPORTANT: No "public" specifier allowed here!?
     *            RMI framework does not seem to work otherwise!
     **************************************************************/
    String simulationGetUUID() throws RemoteException;
    // Session control:
    boolean sessionSuspendRobotThreadIsEnabled(int session_id) throws RemoteException;
    long[] sessionGetData(int session_id) throws RemoteException;
    // support for controlling a test scenario programmatically:
    void sessionResetPreferences(int session_id) throws RemoteException;
    void sessionImportPreferences(int session_id, String pathname) throws RemoteException;
    void sessionImportPreferencesByClass(int session_id, String classname) throws RemoteException;
    void sessionInit(int session_id) throws RemoteException;
    void sessionStart(int session_id) throws RemoteException;
    void sessionStop(int session_id) throws RemoteException;
    void sessionComplete(int session_id) throws RemoteException;
    void sessionTerminate(int session_id) throws RemoteException;
    void sessionRobotExecutionEnded(int session_id) throws RemoteException;
    void sessionPressButton(int session_id, int button_id, int period) throws RemoteException;
    void sessionClap(int volume) throws RemoteException;
    void sessionSetNoiseLevel(int volume) throws RemoteException;
    // Sensor control:
    XSensorInfo sensorRegisterSensor(int session_id, int port_id, int type_id) throws RemoteException;
    XSensorInfo sensorRegisterButton(int session_id, int button_id) throws RemoteException;
    void sensorSwitchLightOn(int session_id, int port_id) throws RemoteException;
    void sensorSwitchLightOff(int session_id, int port_id) throws RemoteException;
    float[] sensorGetValues(int session_id) throws RemoteException;
    // Display control:
    void displayRefresh(int session_id) throws RemoteException;
    void displayClear(int session_id) throws RemoteException;
    void displayClear(int session_id, int y_position) throws RemoteException;
    void displayClear(int session_id, int x_position, int y_position, int count) throws RemoteException;
    void displayDraw(int session_id, char ch, int x_position, int y_position) throws RemoteException;
    void displayDraw(int session_id, int n, int x_position, int y_position) throws RemoteException;
    void displayDraw(int session_id, int n, int x_position, int y_position, int digits) throws RemoteException;
    void displayDraw(int session_id, float fl, int x_position, int y_position) throws RemoteException;
    void displayDraw(int session_id, String text, int x_position, int y_position) throws RemoteException;
    void displayDraw(int session_id, String text, int x_position, int y_position, boolean inverted) throws RemoteException;
    // Sound control:
    int soundGetVolume(int session_id) throws RemoteException;
    void soundSetVolume(int session_id, int volume) throws RemoteException;
    int soundBeep(int session_id) throws RemoteException;
    int soundTwoBeeps(int session_id) throws RemoteException;
    int soundBeepSequence(int session_id) throws RemoteException;
    int soundBeepSequenceUp(int session_id) throws RemoteException;
    int soundBuzz(int session_id) throws RemoteException;
    void soundTone(int session_id, int frequency, int period) throws RemoteException;
    void soundTone(int session_id, int frequency, int period, int volume) throws RemoteException;
    void soundNote(int session_id, int sound_id, int frequency, int period) throws RemoteException;
    // Console control:
    void consoleOpen(int session_id) throws RemoteException;
    void consoleClose(int session_id) throws RemoteException;
    void consolePrint(int session_id, String text) throws RemoteException;
    void consolePrintln(int session_id, String text) throws RemoteException;
    // Motor control:
    float motorGetMaxSpeed(int session_id, int port_id) throws RemoteException;
    void motorStartControl(int session_id, int port_id) throws RemoteException;
    void motorSetToFloatMode(int session_id, int port_id) throws RemoteException;
    float[] motorGetPIDParameters(int session_id, int port_id) throws RemoteException;
    float[] motorGetTachoCounts(int session_id) throws RemoteException;
    void motorSetAllPowerData(int session_id, float[] power) throws RemoteException;
    // Robot control:
    float robotGetDefaultTrackWidth(int session_id) throws RemoteException;
    float robotGetDefaultWheelDiameter(int session_id) throws RemoteException;
    void robotSetTrackWidth(int session_id, float track_width) throws RemoteException;
    void robotSetWheelDiameter(int session_id, float wheel_diameter) throws RemoteException;
    void robotSetMotors(int session_id, int left_motor_port_id, int right_motor_port_id) throws RemoteException;
}
