/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import org.jabotics.pc.swing.*;

import java.io.*;
import java.awt.*;
import javax.swing.*;
import java.util.concurrent.atomic.AtomicBoolean;

/**
 * Beschreiben Sie hier die Klasse XInfoPanel.
 * 
 * @author  Frank Gollnik
 * @version 1.2.1
 */
public class XInfoPanel extends JPanel implements XLogStreamTextArea.ILogListener
{
	class SessionHandler extends BSessionHandler
    {
        @Override
        public String getKey()
        {
            return "XInfoPanel.Session#" + mSession.getID();
        }

        @Override
        public void terminateSession()
        {
            mOutErrPanel.stopLogging();
        }
    }

    // instance variables
    private AtomicBoolean mInitialized = new AtomicBoolean(false);
    private AtomicBoolean mStarted = new AtomicBoolean(false);
    private volatile boolean mOutputBlocked = true;
    private XSession mSession = null;
    private PipedInputStream mISOut = null;
    private PipedInputStream mISErr = null;

    private JTabbedPane mTappedPane;
    private XOutErrStreamPanel mOutErrPanel;

    public XInfoPanel(JTabbedPane tappedPane)
    {
        super(new GridBagLayout());

        mTappedPane = tappedPane;

        mOutErrPanel = new XOutErrStreamPanel();
        Dimension dimMin = new Dimension(210,346);
        Dimension dimMax = new Dimension(600,346);
        mOutErrPanel.setPreferredSize(dimMin);
        mOutErrPanel.setMinimumSize(dimMin);
        mOutErrPanel.setMaximumSize(dimMax);

        GridBagConstraints c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 1.0;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.insets = new Insets(1,1,1,1);
        this.add(mOutErrPanel,c);

        mOutErrPanel.getErrTA().addListener(this);
    }

    public void init()
    {
        if (!mInitialized.compareAndSet(false, true)) return;

        mSession = XSimulationModel.getRobotSession(0);

        mOutErrPanel.getSplitPane().setDividerLocation(1.0);
    }

    public void logEvent(XLogStreamTextArea source)
    {
        if (!mTappedPane.getSelectedComponent().equals(this))
        {
            mTappedPane.setSelectedComponent(this);
        }
    }

    public void startLogging()
    {
        if (!mStarted.compareAndSet(false, true)) return;

        // Wait until UI has been initialized before starting logging (see below).
        SwingUtilities.invokeLater(new Runnable()
        {
        	public void run()
        	{
                // TODO:
        	    // Currently, BlueJ displays JavaFX related error messages
        	    // when starting the simulation by invoking the main method
        	    // of the robot class for debugging purposes. For the moment,
        		// remove everything from the streams during startup of the UI.
        		XSimulation.stopOutErrStreamDrain();

        		PrintStream originalOut = System.out;
                PrintStream originalErr = System.err;

                try
                {
                    PipedOutputStream osOut = new PipedOutputStream();
                    mISOut = new PipedInputStream(osOut);
                    PrintStream psOut = new PrintStream(osOut);
                    System.setOut(psOut);

                    PipedOutputStream osErr = new PipedOutputStream();
                    mISErr = new PipedInputStream(osErr);
                    PrintStream psErr = new PrintStream(osErr);
                    System.setErr(psErr);

                    mSession.addSessionHandler(new SessionHandler());
                }
                catch (IOException ioe)
                {
                    System.setOut(originalOut);
                    System.setErr(originalErr);
                    return;
                }

        		mOutErrPanel.startLogging(mISOut, mISErr);

                System.out.print("www.jabotics.org ((c)");
                System.out.print(XMsg.COPYRIGHT);
                System.out.println(")");
                System.out.print(XMsg.VERSION);
                System.out.print(": ");
                System.out.println(XMsg.VERSION_NUMBER);
                System.out.print(XMsg.AUTHOR);
                System.out.println(": Frank Gollnik");
                System.out.print(XMsg.PUBLISHED_UNDER);
                System.out.println(": GNU GPLv3\n");
        	}
        });
    }
}
