/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 *
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.robot.sim;

import org.jabotics.robot.compile.CS;

/**
 * @author  Frank Gollnik
 * @version 1.2.1
 */
public class XSessionData
{
	static public final int SESSION_STATE_ID = 0;
	static public final int T_ROBOT = 1;
	static public final int DYNAMICS_UPDATE_COUNT = 2;
	static public final int FIELD_LENGTH = 3;
	
	long[] mData;
	
	// Default constructor corresponds to initial session state
	XSessionData()
	{
		mData = new long[FIELD_LENGTH];
		mData[SESSION_STATE_ID] = ESessionState.NEW.ordinal();
		mData[T_ROBOT] = 0;
		mData[DYNAMICS_UPDATE_COUNT] = 0;
	}

	XSessionData(ESessionState session_state, long t_robot, long dynamics_udate_count)
	{
		mData = new long[FIELD_LENGTH];
		mData[SESSION_STATE_ID] = session_state.ordinal();
		mData[T_ROBOT] = t_robot;
		mData[DYNAMICS_UPDATE_COUNT] = dynamics_udate_count;
	}

	XSessionData(long[] data)
	{
		if (CS.ASSERTS_SIM) assert (data.length == FIELD_LENGTH) : "Internal error in XSessionData: Field lenght does not fit";
		mData = data;
	}

	public long[] getData()
	{
		return mData;
	}

	public ESessionState getSesstionState()
	{
		return ESessionState.values()[(int)mData[SESSION_STATE_ID]];
	}

	public long getTRobot()
	{
		return mData[T_ROBOT];
	}

	public long getDynamicsUpdateCount()
	{
		return mData[DYNAMICS_UPDATE_COUNT];
	}
}
