/********************************************************************************************
 *
 * Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
 * 
 * 
 * This file is part of the Java software package org.jabotics (in the following: Jabotics).
 * 
 * Jabotics is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jabotics is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
 * 
 * Jabotics ist Freie Software: Sie können es unter den Bedingungen
 * der GNU General Public License, wie von der Free Software Foundation,
 * Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 * veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 * 
 * Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
 * OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 * Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 * Siehe die GNU General Public License für weitere Details.
 * 
 * Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
 * erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************************/


package org.jabotics.test.robot;

import org.jabotics.robot.de.*;

/**
 * Beschreibe hier die Klasse SensorTest.
 * 
 * @author  Frank Gollnik
 * @version 1.2.1
 */
public class SensorTest extends XRoboter
{
    ////////////////////////////////////////////////////////////////////////////////////////
    // Hier werden die Objekte zur Robotersteuerung bereitgestellt.
    // Die Erzeugung der Objekte übernimmt für uns eine spezielle Klasse XObjektfabrik:
    ////////////////////////////////////////////////////////////////////////////////////////
    static final IMotor linkerMotor = objektfabrik.erzeugeMotor(EMotoranschluss.C);
    static final IMotor rechterMotor = objektfabrik.erzeugeMotor(EMotoranschluss.B);
    static final IPilot pilot = objektfabrik.erzeugePiloten(EMotoranschluss.C, EMotoranschluss.B);
    static final ITastsensor eingabetaste = objektfabrik.erzeugeTastsensor(ETaster.EINGABE);
    static final ITastsensor endetaste = objektfabrik.erzeugeTastsensor(ETaster.ENDE);
    static final ITastsensor linkstaste = objektfabrik.erzeugeTastsensor(ETaster.LINKS);
    static final ITastsensor rechtstaste = objektfabrik.erzeugeTastsensor(ETaster.RECHTS);
    static final ITastsensor tastsensor = objektfabrik.erzeugeTastsensor(ESensoranschluss.S1);
    static final ISensor schallsensor = objektfabrik.erzeugeSchallsensor(ESensoranschluss.S2);
    static final ILichtsensor lichtsensor = objektfabrik.erzeugeLichtsensor(ESensoranschluss.S3);
    //static final ILichtsensor lichtsensor2 = objektfabrik.erzeugeLichtsensor(ESensoranschluss.S4);
    static final ISensor ultraschallsensor = objektfabrik.erzeugeUltraschallsensor(ESensoranschluss.S4);
    static final IUhr uhr = objektfabrik.erzeugeUhr();
    static final IMonitor monitor = objektfabrik.erzeugeMonitor();
    static final ILautsprecher lautsprecher = objektfabrik.erzeugeLautsprecher();
 
     /**
     * Die Methode "main" (das "Hauptprogramm") ist der Startpunkt für das Roboterprogramm.
     * Diese Methode enthält die eigentliche Robotersteuerung, die du selbst programmieren sollst.
     * Ersetze das kleine Beispielprogramm durch deine eigenen Steuerungsbefehle.
     */
    public static void main(String[] args)
    {
        ////////////////////////////////////////////////////////////////////////////////////////
        // Ersetze den folgenden Beispiel-Programmcode durch deine eigene Robotersteuerung:
        ////////////////////////////////////////////////////////////////////////////////////////

        pilot.setzeSollgeschwindigkeit(1f);
        pilot.fahreGerade(VORWAERTS);
        lichtsensor.schalteLichtEin();
        uhr.warteZeit(100);
        //lichtsensor.setzeTraegheit(100);
        //ultraschallsensor.setzeTraegheit(100);
        while (uhr.holeSystemzeit() < 100000)
        {
            uhr.warteZeit(100);
            //float wert = ultraschallsensor.holeMesswert();
            //float aenderung = ultraschallsensor.holeAenderungsrate();
            float wert = lichtsensor.holeMesswert();
            float aenderung = lichtsensor.holeAenderungsrate();
            monitor.schreibe("Messwert:", 0, 0);
            monitor.schreibe(wert, 0, 1);
            monitor.schreibe("Aenderung:", 0, 2);
            monitor.schreibe(aenderung, 0, 3);
        }
        lichtsensor.schalteLichtAus();
        uhr.warteZeit(2000);
    }
}
