/********************************************************************************************
*
* Copyright 2012-2018  Frank Gollnik  <http://www.jabotics.org>
*
* 
* This file is part of the Java software package org.jabotics (in the following: Jabotics).
* 
* Jabotics is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* Jabotics is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with Jabotics.  If not, see <http://www.gnu.org/licenses/>.
* 
* 
* Diese Datei ist Teil des Java-Software-Pakets org.jabotics (im Folgenden kurz: Jabotics).
* 
* Jabotics ist Freie Software: Sie können es unter den Bedingungen
* der GNU General Public License, wie von der Free Software Foundation,
* Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
* veröffentlichten Version, weiterverbreiten und/oder modifizieren.
* 
* Jabotics wird in der Hoffnung, dass es nützlich sein wird, aber
* OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
* Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
* Siehe die GNU General Public License für weitere Details.
* 
* Sie sollten eine Kopie der GNU General Public License zusammen mit Jabotics
* erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
* 
*******************************************************************************************/


package org.jabotics.tools; 


//import java.util.Locale;
import java.net.URL;
//import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import org.jabotics.pc.defs.SIMDefs;
import org.jabotics.pc.swing.XLogStreamTextArea;

import java.util.Map;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import java.util.concurrent.locks.ReentrantLock;
import java.io.File;
import java.io.FileFilter;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;

/**
* Stellt die grundlegende Funktionalität für die Integration der Jabotics-Tools
* in die Entwicklungsumgebungen BlueJ und Eclipse zur Verfügung
* 
* @author  Frank Gollnik
* @version 1.2.1
*/
abstract public class XJaboticsTools
{
    static public final boolean WINDOWS = ('\\' == File.separatorChar); 
    static public final String JAVA_VERSION_NXT = "1.6";
    static public final String JAVA_VERSION_EV3 = "1.7";
    static public final int RMI_PORT = getRMIPort();
    static private int getRMIPort()
    {
        String port = System.getProperty(SIMDefs.KEY_RMI_PORT);
        if (null == port) port = System.getenv(SIMDefs.KEY_RMI_PORT_ENV);
        int n_port = -1;
        if (null != port)
        {
            try {n_port = Integer.parseInt(port);} catch (Exception e) {}
            if (0 == n_port) n_port = 1099; // default port for RMI
        }
        return n_port;
    }

    static final boolean LOGGING = false;
    static final PrintStream LOGFILE;
    static
    {
        PrintStream ps = null;
        if (LOGGING)
        {
            try
            {
                ps = new PrintStream("/home/frank/Temp/jabotics-tools.log");
            }
            catch (FileNotFoundException e)
            {
                System.err.print("File jabotics-tools.log could not be opened.");
            }
        }
        LOGFILE = ps;
    }

    static final String VERSION_NUMBER =    "1.2.1";
    static final String COPYRIGHT =         "2012-2018";

    static final String NXT_BROWSER_ACTION =          "NXT_BROWSER_ACTION";
    static final String NXT_CONSOLE_ACTION =          "NXT_CONSOLE_ACTION";
    static final String NXT_FLASH_ACTION  =           "NXT_FLASH_ACTION";
    static final String NXT_COMPILE_ACTION =          "NXT_COMPILE_ACTION";
    static final String NXT_COMPILEPACKAGE_ACTION =   "NXT_COMPILEPACKAGE_ACTION";
    static final String NXT_UPLOAD_ACTION =           "NXT_UPLOAD_ACTION";
    static final String EV3_CONTROL_ACTION =          "EV3_CONTROL_ACTION";
    static final String EV3_SELECT_ROBOT_ACTION =     "EV3_SELECT_ROBOT_ACTION";
    static final String EV3_SDCARD_ACTION  =          "EV3_SDCARD_ACTION";
    static final String EV3_LIBS_UPLOAD_ACTION =      "EV3_LIBS_UPLOAD_ACTION";
    static final String EV3_COMPILE_ACTION =          "EV3_COMPILE_ACTION";
    static final String EV3_COMPILEPACKAGE_ACTION =   "EV3_COMPILEPACKAGE_ACTION";
    static final String EV3_UPLOAD_ACTION =           "EV3_UPLOAD_ACTION";
    static final String SIMULATE_ACTION =             "SIMULATE_ACTION";
    static final String[] VM_NAMES_WIN =              {"jdk1.9","jdk1.8","jdk1.7","jdk1.6"};
    static final String[] VM_NAMES_LINUX =            {"java-9-oracle","jdk-9-oracle","java-9-openjdk",
                                                      "java-8-oracle","jdk-8-oracle","java-8-openjdk",
                                                      "java-7-oracle","jdk-7-oracle","java-7-sun","java-7-openjdk",
                                                      "java-6-oracle","java-6-sun","java-6-openjdk"};

    // Assign keys, first
    protected String MSG_ROBOT =                  "robot.prompt";
    protected String MSG_ROBOT_EXT =              "robot.ext.title";
    protected String MSG_ROBOT_EXT_INFO =         "robot.ext.info";

    protected String MSG_INFO =                   "info.prompt";
    protected String MSG_VERSION =                "info.version";
    protected String MSG_AUTHOR =                 "info.author";

    protected String MSG_WARNING =                "warn.prompt";
    protected String MSG_ERROR =                  "err.prompt";
    protected String MSG_URL_ERROR =              "err.url";

    protected String MSG_ENV_VAR =                "env.prompt";
    protected String MSG_ENV_VAR_OK =             "env.valid";
    protected String MSG_ENV_VAR_NOT_OK =         "env.invalid";
    protected String MSG_ENV_VAR_NOT_DEFINED =    "env.undefined";
    protected String MSG_DIR_FOUND =              "dir.yes";
    protected String MSG_DIR_NOT_FOUND =          "dir.no";
    protected String MSG_FILE_FOUND =             "file.yes";
    protected String MSG_FILE_NOT_FOUND =         "file.no";
    protected String MSG_NO_JAR_FILE_IN_DIR =     "file.jar.no";
    protected String MSG_JVM_FOUND =              "jvm.yes";
    protected String MSG_JVM_NOT_FOUND =          "jvm.no";
    protected String MSG_NXJ_FOUND =              "nxj.yes";
    protected String MSG_NXJ_NOT_FOUND =          "nxj.no";
    protected String MSG_EV3_FOUND =              "ev3.yes";
    protected String MSG_EV3_NOT_FOUND =          "ev3.no";

    protected String MSG_SHOW_DETAILS =           "details.show";
    protected String MSG_INIT_DETAILS =           "details.init";

    protected String MSG_COMPILING_PACKAGE =      "comp.package";
    protected String MSG_COMPILING =              "comp.prompt";
    protected String MSG_COMPILATION_RUNNING =    "comp.running";
    protected String MSG_COMPILATION_COMPLETED =  "comp.completed";
    protected String MSG_COMPILATION_FAILED =     "comp.failed";

    protected String MSG_JAR =                    "jar.prompt";
    protected String MSG_JAR_COMPLETED =          "jar.completed";
    protected String MSG_JAR_FAILED =             "jar.failed";

    protected String MSG_SIMULATION =             "sim.title";
    protected String MSG_SIM_INIT =               "sim.init";
    protected String MSG_SIM_READY =              "sim.ready.yes";
    protected String MSG_SIM_NOT_READY =          "sim.ready.no";
    protected String MSG_SIM_START =              "sim.prompt";
    protected String MSG_SIM_STARTED =            "sim.started";
    protected String MSG_SIM_RUNNING =            "sim.running";

    protected String MSG_NXT =                    "nxt.prompt";
    protected String MSG_NXT_INIT =               "nxt.init";
    protected String MSG_NXT_READY =              "nxt.ready.yes";
    protected String MSG_NXT_NOT_READY =          "nxt.ready.no";
    protected String MSG_NXT_BROWSER =            "nxt.browser.prompt";
    protected String MSG_NXT_BROWSER_STARTED =    "nxt.browser.started";
    protected String MSG_NXT_BROWSER_RUNNING =    "nxt.browser.running";
    protected String MSG_NXT_CONSOLE =            "nxt.console.prompt";
    protected String MSG_NXT_CONSOLE_STARTED =    "nxt.console.started";
    protected String MSG_NXT_CONSOLE_RUNNING =    "nxt.console.running";
    protected String MSG_NXT_FLASHTOOL =          "nxt.flash.prompt";
    protected String MSG_NXT_FLASHTOOL_STARTED =  "nxt.flash.started";
    protected String MSG_NXT_FLASHTOOL_RUNNING =  "nxt.flash.running";
    protected String MSG_NXT_COMPILEPACKAGE =     "nxt.comp.package";
    protected String MSG_NXT_COMPILE =            "nxt.comp.prompt";
    protected String MSG_NXT_UPLOAD =             "nxt.upload.prompt";
    protected String MSG_NXT_UPLOAD_RUNNING =     "nxt.upload.running";

    protected String MSG_EV3 =                    "ev3.prompt";
    protected String MSG_EV3_INIT =               "ev3.init";
    protected String MSG_EV3_READY =              "ev3.ready.yes";
    protected String MSG_EV3_NOT_READY =          "ev3.ready.no";
    protected String MSG_EV3_CONTROL =            "ev3.control.prompt";
    protected String MSG_EV3_CONTROL_STARTED =    "ev3.control.started";
    protected String MSG_EV3_CONTROL_RUNNING =    "ev3.control.running";
    protected String MSG_EV3_ROBOT =              "ev3.robot.prompt";
    protected String MSG_EV3_ROBOT_ADDRESS =      "ev3.robot.address";
    protected String MSG_EV3_SDCARD =             "ev3.sdcard.prompt";
    protected String MSG_EV3_SDCARD_STARTED =     "ev3.sdcard.started";
    protected String MSG_EV3_SDCARD_RUNNING =     "ev3.sdcard.running";
    protected String MSG_EV3_LIBS =               "ev3.libs.prompt";
    protected String MSG_EV3_LIBS_RUNNING =       "ev3.libs.running";
    protected String MSG_EV3_LIBS_COMPLETED =     "ev3.libs.completed";
    protected String MSG_EV3_LIBS_FAILED =        "ev3.libs.failed";
    protected String MSG_EV3_COMPILEPACKAGE =     "ev3.comp.package";
    protected String MSG_EV3_COMPILE =            "ev3.comp.prompt";
    protected String MSG_EV3_UPLOAD =             "ev3.upload.prompt";
    protected String MSG_EV3_UPLOAD_RUNNING =     "ev3.upload.running";
    protected String MSG_EV3_UPLOAD_COMPLETED =   "ev3.upload.completed";
    protected String MSG_EV3_UPLOAD_FAILED =      "ev3.upload.failed";

    protected String mLanguage;
    protected String mSIMJava;
    protected String mSIMClasspath;
    protected String mProgramRootPath;
    protected String mJaboticsHome;
    protected String mNXTHome;
    protected String mNXTJava;
    protected String mNXTJavac;
    protected String mNXTRuntime;
    protected String mNXTClasspath;
    protected String mNXTClasspathPC;
    protected String mEV3Home;
    protected String mEV3Jar;
    protected String mEV3Java;
    protected String mEV3Javac;
    protected String mEV3Runtime;
    protected String mEV3Classpath;
    protected String mEV3ClasspathPC;
    protected String mEV3LibUser;
    protected ArrayList<File> mEV3UserlibFiles = new ArrayList<File>();
    protected boolean mReadyNXT;
    protected boolean mReadyEV3;
    protected boolean mReadySIM;
    protected String mInitDetails;
    protected String mInfo;
    protected ProcessBuilder mPB;
    protected ArrayList<String> mCommandList = new ArrayList<String>();
    protected XClassFileFilter mClassFileFilter = new XClassFileFilter();

    static XReentrantLock sConnectionLock = new XReentrantLock();
    static XReentrantLock sCompileLock = new XReentrantLock();

    abstract protected String  getLanguage();
    abstract protected String  getSimLibDirPath();
    abstract protected String  getJaboticsToolsPath();
    abstract protected boolean tryCompileClass(XActionTarget target);
    abstract protected String  getPropertyString(String key, String default_value);
    abstract protected void    setPropertyString(String key, String value);
    abstract protected void    showMessageDialog(String message);
    abstract protected String  showInputDialog(String message, String initialSelection);

    private void initMessages()
    {
        if (LOGGING) LOGFILE.println("XJaboticsTools.initMessages()");
        // do not allow the initialization of the extension to crash due to a problem with the internationalization
        try
        {
            Locale currentLocale = new Locale(mLanguage);

            ResourceBundle messages = ResourceBundle.getBundle("resources/properties/jabotics-tools-messages", currentLocale);
   
            // Replace key by translation
            MSG_ROBOT =                  messages.getString(MSG_ROBOT);
            MSG_ROBOT_EXT =              messages.getString(MSG_ROBOT_EXT);
            MSG_ROBOT_EXT_INFO =         messages.getString(MSG_ROBOT_EXT_INFO);

            MSG_VERSION =                messages.getString(MSG_VERSION);
            MSG_AUTHOR =                 messages.getString(MSG_AUTHOR);

            MSG_INFO =                   messages.getString(MSG_INFO);
            MSG_WARNING =                messages.getString(MSG_WARNING);
            MSG_ERROR =                  messages.getString(MSG_ERROR);
            MSG_URL_ERROR =              messages.getString(MSG_URL_ERROR);

            MSG_ENV_VAR =                messages.getString(MSG_ENV_VAR);
            MSG_ENV_VAR_OK =             messages.getString(MSG_ENV_VAR_OK);
            MSG_ENV_VAR_NOT_OK =         messages.getString(MSG_ENV_VAR_NOT_OK);
            MSG_ENV_VAR_NOT_DEFINED =    messages.getString(MSG_ENV_VAR_NOT_DEFINED);
            MSG_DIR_FOUND =              messages.getString(MSG_DIR_FOUND);
            MSG_DIR_NOT_FOUND =          messages.getString(MSG_DIR_NOT_FOUND);
            MSG_FILE_FOUND =             messages.getString(MSG_FILE_FOUND);
            MSG_FILE_NOT_FOUND =         messages.getString(MSG_FILE_NOT_FOUND);
            MSG_NO_JAR_FILE_IN_DIR =     messages.getString(MSG_NO_JAR_FILE_IN_DIR);
            MSG_JVM_FOUND =              messages.getString(MSG_JVM_FOUND);
            MSG_JVM_NOT_FOUND =          messages.getString(MSG_JVM_NOT_FOUND);
            MSG_NXJ_FOUND =              messages.getString(MSG_NXJ_FOUND);
            MSG_NXJ_NOT_FOUND =          messages.getString(MSG_NXJ_NOT_FOUND);
            MSG_EV3_FOUND =              messages.getString(MSG_EV3_FOUND);
            MSG_EV3_NOT_FOUND =          messages.getString(MSG_EV3_NOT_FOUND);

            MSG_SHOW_DETAILS =           messages.getString(MSG_SHOW_DETAILS);
            MSG_INIT_DETAILS =           messages.getString(MSG_INIT_DETAILS);

            MSG_COMPILING_PACKAGE =      messages.getString(MSG_COMPILING_PACKAGE);
            MSG_COMPILING =              messages.getString(MSG_COMPILING);
            MSG_COMPILATION_RUNNING =    messages.getString(MSG_COMPILATION_RUNNING);
            MSG_COMPILATION_COMPLETED =  messages.getString(MSG_COMPILATION_COMPLETED);
            MSG_COMPILATION_FAILED =     messages.getString(MSG_COMPILATION_FAILED);

            MSG_JAR =                    messages.getString(MSG_JAR);
            MSG_JAR_COMPLETED =          messages.getString(MSG_JAR_COMPLETED);
            MSG_JAR_FAILED =             messages.getString(MSG_JAR_FAILED);

            MSG_SIMULATION =             messages.getString(MSG_SIMULATION);
            MSG_SIM_INIT =               messages.getString(MSG_SIM_INIT);
            MSG_SIM_READY =              messages.getString(MSG_SIM_READY);
            MSG_SIM_NOT_READY =          messages.getString(MSG_SIM_NOT_READY);
            MSG_SIM_START =              messages.getString(MSG_SIM_START);
            MSG_SIM_STARTED =            messages.getString(MSG_SIM_STARTED);
            MSG_SIM_RUNNING =            messages.getString(MSG_SIM_RUNNING);

            MSG_NXT =                    messages.getString(MSG_NXT);
            MSG_NXT_INIT =               messages.getString(MSG_NXT_INIT);
            MSG_NXT_READY =              messages.getString(MSG_NXT_READY);
            MSG_NXT_NOT_READY =          messages.getString(MSG_NXT_NOT_READY);
            MSG_NXT_BROWSER =            messages.getString(MSG_NXT_BROWSER);
            MSG_NXT_BROWSER_STARTED =    messages.getString(MSG_NXT_BROWSER_STARTED);
            MSG_NXT_BROWSER_RUNNING =    messages.getString(MSG_NXT_BROWSER_RUNNING);
            MSG_NXT_CONSOLE =            messages.getString(MSG_NXT_CONSOLE);
            MSG_NXT_CONSOLE_STARTED =    messages.getString(MSG_NXT_CONSOLE_STARTED);
            MSG_NXT_CONSOLE_RUNNING =    messages.getString(MSG_NXT_CONSOLE_RUNNING);
            MSG_NXT_FLASHTOOL =          messages.getString(MSG_NXT_FLASHTOOL);
            MSG_NXT_FLASHTOOL_STARTED =  messages.getString(MSG_NXT_FLASHTOOL_STARTED);
            MSG_NXT_FLASHTOOL_RUNNING =  messages.getString(MSG_NXT_FLASHTOOL_RUNNING);
            MSG_NXT_COMPILEPACKAGE =     messages.getString(MSG_NXT_COMPILEPACKAGE);
            MSG_NXT_COMPILE =            messages.getString(MSG_NXT_COMPILE);
            MSG_NXT_UPLOAD =             messages.getString(MSG_NXT_UPLOAD);
            MSG_NXT_UPLOAD_RUNNING =     messages.getString(MSG_NXT_UPLOAD_RUNNING);

            MSG_EV3 =                    messages.getString(MSG_EV3);
            MSG_EV3_INIT =               messages.getString(MSG_EV3_INIT);
            MSG_EV3_READY =              messages.getString(MSG_EV3_READY);
            MSG_EV3_NOT_READY =          messages.getString(MSG_EV3_NOT_READY);
            MSG_EV3_CONTROL =            messages.getString(MSG_EV3_CONTROL);
            MSG_EV3_CONTROL_STARTED =    messages.getString(MSG_EV3_CONTROL_STARTED);
            MSG_EV3_CONTROL_RUNNING =    messages.getString(MSG_EV3_CONTROL_RUNNING);
            MSG_EV3_ROBOT =              messages.getString(MSG_EV3_ROBOT);
            MSG_EV3_ROBOT_ADDRESS =      messages.getString(MSG_EV3_ROBOT_ADDRESS);
            MSG_EV3_SDCARD =             messages.getString(MSG_EV3_SDCARD);
            MSG_EV3_SDCARD_STARTED =     messages.getString(MSG_EV3_SDCARD_STARTED);
            MSG_EV3_SDCARD_RUNNING =     messages.getString(MSG_EV3_SDCARD_RUNNING);
            MSG_EV3_LIBS =               messages.getString(MSG_EV3_LIBS);
            MSG_EV3_LIBS_RUNNING =       messages.getString(MSG_EV3_LIBS_RUNNING);
            MSG_EV3_LIBS_COMPLETED =     messages.getString(MSG_EV3_LIBS_COMPLETED);
            MSG_EV3_LIBS_FAILED =        messages.getString(MSG_EV3_LIBS_FAILED);
            MSG_EV3_COMPILEPACKAGE =     messages.getString(MSG_EV3_COMPILEPACKAGE);
            MSG_EV3_COMPILE =            messages.getString(MSG_EV3_COMPILE);
            MSG_EV3_UPLOAD =             messages.getString(MSG_EV3_UPLOAD);
            MSG_EV3_UPLOAD_RUNNING =     messages.getString(MSG_EV3_UPLOAD_RUNNING);
            MSG_EV3_UPLOAD_COMPLETED =   messages.getString(MSG_EV3_UPLOAD_COMPLETED);
            MSG_EV3_UPLOAD_FAILED =      messages.getString(MSG_EV3_UPLOAD_FAILED);
        }
        catch (Exception e)
        {
            System.out.println ("Robot extension: Internationialization of messages failed. Details:\n" + e.getMessage());
        }
    }

    public void startup()
    {
        if (LOGGING) LOGFILE.println("XJaboticsTools.startup(): start");

        switch (getLanguage())
        {
        case "de":
            mLanguage = "de";
            break;
        default:
           mLanguage = "en";
        }

        initMessages();

        mReadyNXT = true;
        mReadyEV3 = true;
        mReadySIM = true;
        //mBereit = true;

        mPB = new ProcessBuilder(mCommandList);

        // Lade die benötigen Umgebungsvariablen
        Map<String,String> env = System.getenv();

        StringBuilder sb = new StringBuilder();


        sb.append("---------------------------------------------------------------\n");
        sb.append(MSG_SIM_INIT);
        sb.append('\n');
        sb.append("---------------------------------------------------------------\n");

        if (WINDOWS)
        {
            mProgramRootPath = "C:\\Program Files (x86)";
            File rootDir = new File(mProgramRootPath);
            if (!rootDir.exists())
            {
                mProgramRootPath = "C:\\Program Files";
            }
        }
        else
        {
            mProgramRootPath = "/usr/share";
        }

        mSIMJava = "";
        String simJavaHome = null;
        simJavaHome = getSuitableJavaPath();
        if (null == simJavaHome)
        {
            String javaHome = env.get("JAVA_HOME");
            if (null == javaHome)
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_ENV_VAR_NOT_DEFINED);
                sb.append(" JAVA_HOME\n");
                sb.append(MSG_JVM_NOT_FOUND);
                sb.append('\n');
            }
            else
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_ENV_VAR_NOT_OK);
                sb.append(" JAVA_HOME = ");
                sb.append(javaHome);
                sb.append('\n');
            }
            mReadySIM = false;
        }
        if (mReadySIM)
        {
            mSIMJava = simJavaHome + File.separatorChar + "bin" + File.separatorChar + ((WINDOWS) ? "java.exe" : "java");
            if (!(new File(mSIMJava)).isFile())
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_FILE_NOT_FOUND);
                sb.append(' ');
                sb.append(mSIMJava);
                sb.append('\n');
                mReadySIM = false;
            }
        }

        String simLibDirPath = getSimLibDirPath();
        File simLibDir = new File(simLibDirPath);
        if (!simLibDir.isDirectory())
        {
            sb.append(MSG_ERROR);
            sb.append(" - ");
            sb.append(MSG_DIR_NOT_FOUND);
            sb.append(' ');
            sb.append(simLibDirPath);
            sb.append('\n');
            mReadySIM = false;
        }
        else
        {
            sb.append(MSG_DIR_FOUND);
            sb.append(' ');
            sb.append(simLibDirPath);
            sb.append('\n');
            mSIMClasspath = buildClasspath(simLibDir);
            if (null == mSIMClasspath)
            {
                sb.append(MSG_ERROR);
                sb.append(": ");
                sb.append(MSG_NO_JAR_FILE_IN_DIR);
                sb.append('\n');
                mReadySIM = false;
            }
            else if (!mSIMClasspath.contains("jabotics-sim.jar"))
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_FILE_NOT_FOUND);
                sb.append(" jabotics-sim.jar\n");
                mReadySIM = false;
            }
            else
            {
                sb.append(MSG_FILE_FOUND);
                sb.append(" jabotics-sim.jar\n");
            }
        }

        String envJaboticsHome = env.get("JABOTICS_HOME");
        if (null == envJaboticsHome)
        {
            mJaboticsHome = mProgramRootPath + File.separatorChar + ((WINDOWS) ? "Jabotics" : "jabotics");
        }
        else
        {
            mJaboticsHome = envJaboticsHome;
        }
        File jaboticsHomeDir = new File(mJaboticsHome);
        if (!jaboticsHomeDir.exists() || !jaboticsHomeDir.isDirectory())
        {
            if (null == envJaboticsHome)
            {
                mJaboticsHome = null;
                sb.append(MSG_ENV_VAR_NOT_DEFINED);
                sb.append(" JABOTICS_HOME\n");
            }
            else
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_ENV_VAR_NOT_OK);
                sb.append(" JABOTICS_HOME = ");
                sb.append(envJaboticsHome);
                sb.append('\n');
            }
        }
        
        
        sb.append("---------------------------------------------------------------\n");
        sb.append(MSG_NXT_INIT);
        sb.append('\n');
        sb.append("---------------------------------------------------------------\n");

        mNXTHome = env.get("NXJ_HOME");
        mNXTRuntime = "";
        mNXTClasspathPC = "";
        String nxtLibPCPath;
        String nxtLibNXTPath;
        String nxtLibUserPath;
        File nxtLibPCDir;
        File nxtLibNXTDir;
        File nxtLibUserDir;
        if (null == mNXTHome)
        {
            mNXTHome = getSuitableLeJOSHome("leJOS_NXJ");
            if (null == mNXTHome)
            {
                sb.append(MSG_ENV_VAR_NOT_DEFINED);
                sb.append(" NXJ_HOME\n");
                sb.append(MSG_NXJ_NOT_FOUND);
                sb.append('\n');
                mReadyNXT = false;
            }
            else
            {
                sb.append(MSG_ENV_VAR_NOT_DEFINED);
                sb.append(" NXJ_HOME\n");
                sb.append(MSG_NXJ_FOUND);
                sb.append(' ');
                sb.append(mNXTHome);
                sb.append('\n');
            }
        }
        else if (!(new File(mNXTHome)).isDirectory())
        {
            sb.append(MSG_ERROR);
            sb.append(" - ");
            sb.append(MSG_ENV_VAR_NOT_OK);
            sb.append(" NXJ_HOME = ");
            sb.append(mNXTHome);
            sb.append('\n');
            mReadyNXT = false;
        }
        else
        {
            sb.append(MSG_ENV_VAR_OK);
            sb.append(" NXJ_HOME = ");
            sb.append(mNXTHome);
            sb.append('\n');
        }

        if (mReadyNXT)
        {
            nxtLibNXTPath = mNXTHome + File.separatorChar + "lib" + File.separatorChar + "nxt";
            nxtLibNXTDir = new File(nxtLibNXTPath);
            if (nxtLibNXTDir.isDirectory())
            {
                sb.append(MSG_DIR_FOUND);
                sb.append(' ');
                sb.append(nxtLibNXTPath);
                sb.append('\n');
                mNXTRuntime = nxtLibNXTPath + File.separatorChar + "classes.jar";
                if (!(new File(mNXTRuntime)).isFile())
                {
                    sb.append(MSG_ERROR);
                    sb.append(" - ");
                    sb.append(MSG_FILE_NOT_FOUND);
                    sb.append(' ');
                    sb.append(mNXTRuntime);
                    sb.append('\n');
                    mReadyNXT = false;
                }
            }
            else
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_DIR_NOT_FOUND);
                sb.append(' ');
                sb.append(nxtLibNXTPath);
                sb.append('\n');
                mReadyNXT = false;
            }
   
            nxtLibPCPath = mNXTHome + File.separatorChar + "lib" + File.separatorChar + "pc";
            nxtLibPCDir = new File(nxtLibPCPath);
            if (nxtLibPCDir.isDirectory())
            {
                sb.append(MSG_DIR_FOUND);
                sb.append(' ');
                sb.append(nxtLibPCPath);
                sb.append('\n');
                mNXTClasspathPC = buildClasspath(nxtLibPCDir);
                if (null == mNXTClasspathPC)
                {
                    sb.append(MSG_ERROR);
                    sb.append(": ");
                    sb.append(MSG_NO_JAR_FILE_IN_DIR);
                    sb.append('\n');
                    mReadyNXT = false;
                }
            }
            else
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_DIR_NOT_FOUND);
                sb.append(' ');
                sb.append(nxtLibPCPath);
                sb.append('\n');
                mReadyNXT = false;
            }
   
            nxtLibUserPath = mNXTHome + File.separatorChar + "lib" + File.separatorChar + "userlib";
            nxtLibUserDir = new File(nxtLibUserPath);
            if (nxtLibUserDir.isDirectory())
            {
                sb.append(MSG_DIR_FOUND);
                sb.append(' ');
                sb.append(nxtLibUserPath);
                sb.append('\n');
                mNXTClasspath = buildClasspath(nxtLibUserDir);
                if (null == mNXTClasspath)
                {
                    sb.append(MSG_WARNING);
                    sb.append(": ");
                    sb.append(MSG_NO_JAR_FILE_IN_DIR);
                    sb.append('\n');
                }
            }
            else
            {
                sb.append(MSG_WARNING);
                sb.append(" - ");
                sb.append(MSG_DIR_NOT_FOUND);
                sb.append(' ');
                sb.append(nxtLibUserPath);
                sb.append('\n');
            }
        }

        String nxjJavaHome = env.get("LEJOS_NXJ_JAVA_HOME");
        mNXTJava = "";
        mNXTJavac = "";
        if (null == nxjJavaHome)
        {
            nxjJavaHome = simJavaHome;

            if (null == nxjJavaHome)
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_ENV_VAR_NOT_DEFINED);
                sb.append(" LEJOS_NXJ_JAVA_HOME\n");
                sb.append(MSG_JVM_NOT_FOUND);
                sb.append('\n');
                mReadyNXT = false;
            }
            else
            {
                sb.append(MSG_ENV_VAR_NOT_DEFINED);
                sb.append(" LEJOS_NXJ_JAVA_HOME\n");
                sb.append(MSG_JVM_FOUND);
                sb.append(' ');
                sb.append(nxjJavaHome);
                sb.append('\n');
            }
        }
        else if (!(new File(nxjJavaHome)).isDirectory())
        {
            sb.append(MSG_ERROR);
            sb.append(" - ");
            sb.append(MSG_ENV_VAR_NOT_OK);
            sb.append(" LEJOS_NXJ_JAVA_HOME = ");
            sb.append(nxjJavaHome);
            sb.append('\n');
            mReadyNXT = false;
        }
        else
        {
            sb.append(MSG_ENV_VAR_OK);
            sb.append(" LEJOS_NXJ_JAVA_HOME = ");
            sb.append(nxjJavaHome);
            sb.append('\n');
        }
        if (mReadyNXT)
        {
            mNXTJava = nxjJavaHome + File.separatorChar + "bin" + File.separatorChar + ((WINDOWS) ? "java.exe" : "java");
            if (!(new File(mNXTJava)).isFile())
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_FILE_NOT_FOUND);
                sb.append(' ');
                sb.append(mNXTJava);
                sb.append('\n');
                mReadyNXT = false;
            }

            mNXTJavac = nxjJavaHome + File.separatorChar + "bin" + File.separatorChar + ((WINDOWS) ? "javac.exe" : "javac");
            if (!(new File(mNXTJavac)).isFile())
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_FILE_NOT_FOUND);
                sb.append(' ');
                sb.append(mNXTJavac);
                sb.append('\n');
                mReadyNXT = false;
            }
        }


        sb.append("---------------------------------------------------------------\n");
        sb.append(MSG_EV3_INIT);
        sb.append('\n');
        sb.append("---------------------------------------------------------------\n");

        mEV3Home = env.get("EV3_HOME");
        mEV3ClasspathPC = "";
        String ev3ClasspathUserlib = "";
        String ev3LibEV3;
        String ev3LibPC;
        mEV3LibUser = "";
        File ev3LibPCDir;
        File ev3LibEV3Dir;
        File ev3LibUserDir;
        if (null == mEV3Home)
        {
            mEV3Home = getSuitableLeJOSHome("leJOS_EV3");
            if (null == mEV3Home)
            {
                sb.append(MSG_ENV_VAR_NOT_DEFINED);
                sb.append(" EV3_HOME\n");
                sb.append(MSG_EV3_NOT_FOUND);
                sb.append('\n');
                mReadyEV3 = false;
            }
            else
            {
                sb.append(MSG_ENV_VAR_NOT_DEFINED);
                sb.append(" EV3_HOME\n");
                sb.append(MSG_EV3_FOUND);
                sb.append(' ');
                sb.append(mEV3Home);
                sb.append('\n');
            }
        }
        else if (!(new File(mEV3Home)).isDirectory())
        {
            sb.append(MSG_ERROR);
            sb.append(" - ");
            sb.append(MSG_ENV_VAR_NOT_OK);
            sb.append(" EV3_HOME = ");
            sb.append(mEV3Home);
            sb.append('\n');
            mReadyEV3 = false;
        }
        else
        {
            sb.append(MSG_ENV_VAR_OK);
            sb.append(" EV3_HOME = ");
            sb.append(mEV3Home);
            sb.append('\n');
        }

        if (mReadyEV3)
        {
            ev3LibEV3 = mEV3Home + File.separatorChar + "lib" + File.separatorChar + "ev3";
            ev3LibEV3Dir = new File(ev3LibEV3);
            if (ev3LibEV3Dir.isDirectory())
            {
                sb.append(MSG_DIR_FOUND);
                sb.append(' ');
                sb.append(ev3LibEV3);
                sb.append('\n');
                mEV3Classpath = buildClasspath(ev3LibEV3Dir);
                if (null == mEV3Classpath)
                {
                    sb.append(MSG_ERROR);
                    sb.append(": ");
                    sb.append(MSG_NO_JAR_FILE_IN_DIR);
                    sb.append('\n');
                    mReadyEV3 = false;
                }
            }
            else
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_DIR_NOT_FOUND);
                sb.append(' ');
                sb.append(ev3LibEV3);
                sb.append('\n');
                mReadyEV3 = false;
            }
        }

        if (mReadyEV3)
        {
            ev3LibPC = mEV3Home + File.separatorChar + "lib" + File.separatorChar + "pc";
            ev3LibPCDir = new File(ev3LibPC);
            if (ev3LibPCDir.isDirectory())
            {
                sb.append(MSG_DIR_FOUND);
                sb.append(' ');
                sb.append(ev3LibPC);
                sb.append('\n');
                mEV3ClasspathPC = buildClasspath(ev3LibPCDir);
                if (null == mEV3ClasspathPC)
                {
                    sb.append(MSG_ERROR);
                    sb.append(": ");
                    sb.append(MSG_NO_JAR_FILE_IN_DIR);
                    sb.append('\n');
                    mReadyEV3 = false;
                }
                else
                {
                    // some PC tools require binaries from ev3classes.jar
                    mEV3ClasspathPC = mEV3ClasspathPC + File.pathSeparator + mEV3Classpath;
                }
            }
            else
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_DIR_NOT_FOUND);
                sb.append(' ');
                sb.append(ev3LibPC);
                sb.append('\n');
                mReadyEV3 = false;
            }
        }

        if (mReadyEV3)
        {
            mEV3LibUser = mEV3Home + File.separatorChar + "lib" + File.separatorChar + "userlib";
            ev3LibUserDir = new File(mEV3LibUser);
            if (ev3LibUserDir.isDirectory())
            {
                sb.append(MSG_DIR_FOUND);
                sb.append(' ');
                sb.append(mEV3LibUser);
                sb.append('\n');
                ev3ClasspathUserlib = buildClasspath(ev3LibUserDir,mEV3UserlibFiles);
                if (null == ev3ClasspathUserlib)
                {
                    sb.append(MSG_WARNING);
                    sb.append(": ");
                    sb.append(MSG_NO_JAR_FILE_IN_DIR);
                    sb.append('\n');
                }
                else
                {
                    mEV3Classpath = mEV3Classpath + File.pathSeparator + ev3ClasspathUserlib;
                }
            }
            else
            {
                sb.append(MSG_WARNING);
                sb.append(" - ");
                sb.append(MSG_DIR_NOT_FOUND);
                sb.append(' ');
                sb.append(mEV3LibUser);
                sb.append('\n');
            }
        }

        String ev3JavaHome = env.get("LEJOS_EV3_JAVA_HOME");
        mEV3Java = "";
        mEV3Javac = "";
        mEV3Jar = "";
        mEV3Runtime = "";  // Dummy runtime to avoid compiler warning for missing bootclasspath
        if (null == ev3JavaHome)
        {
            ev3JavaHome = simJavaHome;

            if (null == ev3JavaHome)
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_ENV_VAR_NOT_DEFINED);
                sb.append(" LEJOS_EV3_JAVA_HOME\n");
                sb.append(MSG_JVM_NOT_FOUND);
                sb.append('\n');
                mReadyEV3 = false;
            }
            else
            {
                sb.append(MSG_ENV_VAR_NOT_DEFINED);
                sb.append(" LEJOS_EV3_JAVA_HOME\n");
                sb.append(MSG_JVM_FOUND);
                sb.append(' ');
                sb.append(ev3JavaHome);
                sb.append('\n');
            }
        }
        else if (!(new File(ev3JavaHome)).isDirectory())
        {
            sb.append(MSG_ERROR);
            sb.append(" - ");
            sb.append(MSG_ENV_VAR_NOT_OK);
            sb.append(" LEJOS_EV3_JAVA_HOME = ");
            sb.append(ev3JavaHome);
            sb.append('\n');
            mReadyEV3 = false;
        }
        else
        {
            sb.append(MSG_ENV_VAR_OK);
            sb.append(" LEJOS_EV3_JAVA_HOME = ");
            sb.append(ev3JavaHome);
            sb.append('\n');
        }
        if (mReadyEV3)
        {
            mEV3Java = ev3JavaHome + File.separatorChar + "bin" + File.separatorChar + ((WINDOWS) ? "java.exe" : "java");
            if (!(new File(mEV3Java)).isFile())
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_FILE_NOT_FOUND);
                sb.append(' ');
                sb.append(mEV3Java);
                sb.append('\n');
                mReadyEV3 = false;
            }

            mEV3Javac = ev3JavaHome + File.separatorChar + "bin" + File.separatorChar + ((WINDOWS) ? "javac.exe" : "javac");
            if (!(new File(mEV3Javac)).isFile())
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_FILE_NOT_FOUND);
                sb.append(' ');
                sb.append(mEV3Javac);
                sb.append('\n');
                mReadyEV3 = false;
            }

            mEV3Jar = ev3JavaHome + File.separatorChar + "bin" + File.separatorChar + ((WINDOWS) ? "jar.exe" : "jar");
            if (!(new File(mEV3Jar)).isFile())
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_FILE_NOT_FOUND);
                sb.append(' ');
                sb.append(mEV3Jar);
                sb.append('\n');
                mReadyEV3 = false;
            }

            mEV3Runtime = ev3JavaHome + File.separatorChar + "jre" + File.separatorChar + "lib" + File.separatorChar + "rt.jar";
            if (!(new File(mEV3Runtime)).isFile())
            {
                sb.append(MSG_ERROR);
                sb.append(" - ");
                sb.append(MSG_FILE_NOT_FOUND);
                sb.append(' ');
                sb.append(mEV3Runtime);
                sb.append('\n');
                mReadyEV3 = false;
            }
        }

        mInitDetails = sb.toString();
        sb.setLength(0);

        sb.append(MSG_VERSION);
        sb.append(": ");
        sb.append(VERSION_NUMBER);
        sb.append('\n');
        sb.append(MSG_AUTHOR);
        sb.append(": Frank Gollnik\n");
        sb.append("www.jabotics.org ((c)");
        sb.append(COPYRIGHT);
        sb.append(")\n");
        sb.append(mReadySIM ? MSG_SIM_READY : MSG_SIM_NOT_READY);
        sb.append('\n');
        sb.append(mReadyNXT ? MSG_NXT_READY : MSG_NXT_NOT_READY);
        sb.append('\n');
        sb.append(mReadyEV3 ? MSG_EV3_READY : MSG_EV3_NOT_READY);

        mInfo = sb.toString();
        sb.setLength(0);

        if (LOGGING)
        {
            LOGFILE.println("XJaboticsTools.mReadyNXT = " + mReadyNXT);
            LOGFILE.println("XJaboticsTools.mReadyEV3 = " + mReadyEV3);
            LOGFILE.println("XJaboticsTools.mReadySIM = " + mReadySIM);
            LOGFILE.println("XJaboticsTools.startup(): end");
        }
    }

    static private class DirectoryFilterByPrefix implements FileFilter
    {
        String mPrefix;

        DirectoryFilterByPrefix(String prefix)
        {
            mPrefix = prefix;
        }

        public boolean accept(File file)
        {
            return file.isDirectory() && file.getName().startsWith(mPrefix);
        }
    }

    private String getSuitableJavaPath()
    {
        if (WINDOWS)
        {
            // Use BlueJ's JDK, if available:
            File jvmDir = new File(mProgramRootPath + "\\BlueJ\\jdk");
            if (jvmDir.exists() && jvmDir.isDirectory())
            {
                File toolsJar = new File(jvmDir.getAbsolutePath() + "\\lib\\tools.jar");
                if (toolsJar.exists() && toolsJar.isFile()) return jvmDir.getAbsolutePath();
            }

            // Use JDK defined by JAVA_HOME, if available:
            String simJavaHome = System.getenv("JAVA_HOME");
            if (null != simJavaHome)
            {
                jvmDir = new File(simJavaHome);
                return (jvmDir.exists() && jvmDir.isDirectory()) ? simJavaHome : null;
            }

            // Look for appropriate JDK at default location:
            jvmDir = new File(mProgramRootPath + "\\Java");
            if (!jvmDir.exists() || !jvmDir.isDirectory()) return null;

            for (int i = 0;  i < VM_NAMES_WIN.length;  ++i)
            {
                File[] files = jvmDir.listFiles(new DirectoryFilterByPrefix(VM_NAMES_WIN[i]));
                for (File file : files)
                {
                    File toolsJar = new File(file.getAbsolutePath() + "\\lib\\tools.jar");
                    if (toolsJar.exists() && toolsJar.isFile()) return file.getAbsolutePath();
                }
            }
            return null;
        }
        else
        {
            File jvmDir;

            // Use JDK defined by JAVA_HOME, if available:
            String simJavaHome = System.getenv("JAVA_HOME");
            if (null != simJavaHome)
            {
                jvmDir = new File(simJavaHome);
                return (jvmDir.exists() && jvmDir.isDirectory()) ? simJavaHome : null;
            }

            // Look for appropriate JDK at default location:
            jvmDir = new File("/usr/lib/jvm");
            if (!jvmDir.exists() || !jvmDir.isDirectory()) return null;

            for (int i = 0;  i < VM_NAMES_LINUX.length;  ++i)
            {
                File[] files = jvmDir.listFiles(new DirectoryFilterByPrefix(VM_NAMES_LINUX[i]));
                for (File file : files)
                {
                    File toolsJar = new File(file.getAbsolutePath() + "/lib/tools.jar");
                    if (toolsJar.exists() && toolsJar.isFile()) return file.getAbsolutePath();
                }
            }

            return null;
        }
    }

    private String getSuitableLeJOSHome(String leJOS_Version)
    {
        if (null != mJaboticsHome)
        {
            String lejosHome = getSuitableLeJOSHome(mJaboticsHome, leJOS_Version);
            if (null != lejosHome) return lejosHome;
        }

        return getSuitableLeJOSHome(WINDOWS ? mProgramRootPath : "/opt", leJOS_Version);
    }

    static private String getSuitableLeJOSHome(String dirName, String leJOS_Version)
    {
        File dir = new File(dirName);
        if (!dir.exists() || !dir.isDirectory()) return null;

        String home = dirName + File.separatorChar + leJOS_Version;
        File homeDir = new File(home);
        if (homeDir.exists() && homeDir.isDirectory()) return home;

        File[] files = dir.listFiles(new DirectoryFilterByPrefix(leJOS_Version));
        for (File file : files)
        {
            return file.getAbsolutePath();
        }

        return null;
    }

    static private String buildClasspath(File dir)
    {
        ArrayList<File> fileList = new ArrayList<File>();
        return buildClasspath(dir, fileList);
    }

    static private String buildClasspath(File dir, ArrayList<File> fileList)
    {
        getJavaBinaries(fileList, dir, true);
        if (fileList.isEmpty()) return null;
        StringBuilder sb = new StringBuilder();
        for(File item : fileList)
        {
            sb.append(File.pathSeparator);  // ";" für Windows und ":" für Unix
            sb.append(decoratedPath(item.getAbsolutePath()));
        }
        sb.deleteCharAt(0);
        return sb.toString();
    }

    static private void getJavaBinaries(ArrayList<File> fileList, File dir, boolean recursive)
    {
        File[] files = dir.listFiles();
        for (File item : files)
        {
            if (item.isFile())
            {
                String lowerCaseName = item.getName().toLowerCase();
                if (lowerCaseName.endsWith(".jar") || lowerCaseName.endsWith(".class"))
                {
                    fileList.add(item);
                }
            }
            else
            {
                if (recursive && item.isDirectory()) getJavaBinaries(fileList, item, recursive);
            }
        }
    }

    static private String decoratedPath(String path)
    {
        return WINDOWS ? ("\"" + path + "\"") : path;
    }


    public String getToolsVersion()
    { 
        return (VERSION_NUMBER);  
    }

    public String getToolsName()
    { 
        return (MSG_ROBOT_EXT);  
    }

    public void terminate()
    {
        //System.out.println ("Robot extension terminates");
    }
   
    public String getDescription()
    {
        return (MSG_ROBOT_EXT_INFO);
    }

    public URL getToolsURL()
    {
        try
        {
            return new URL("http://www.jabotics.org");
        }
        catch ( Exception eee )
        {
            // The link is either dead or otherwise unreachable
            System.out.println (MSG_URL_ERROR + ":\n" + eee.getMessage());
            return null;
        }
    }
   

    static class XReentrantLock extends ReentrantLock
    {
        // Eigene abgeleitete Klasse sichert Zugriff auf protected-Methode getOwner
        XReentrantLock()
        {
            super();
        }

        String getOwnerName()
        {
            Thread thread = this.getOwner();
            if (null == thread) return null;
            try
            {
                return thread.getName(); // Vermutlich kann hier gar nichts schiefgehen.
            }
            catch (Exception e) {}
            return null;
        }
    }

    private class XClassFileFilter implements XLogStreamTextArea.ILogFilter
    {
        private Vector<String> mClassFiles = new Vector<String>();

        void init()
        {
            mClassFiles.clear();
        }

        Iterator<String> getClassFileIterator()
        {
            return mClassFiles.iterator();
        }

        @Override
        public boolean blockLine(XLogStreamTextArea source, String line)
        {
            boolean block = line.startsWith("[");
            if (line.startsWith("[wrote RegularFileObject[") && line.endsWith(".class]]"))
            {
                mClassFiles.addElement(line.substring(25, line.length()-2));
            }
            return block;
        }
    }

    static class XActionTarget
    {
        final File   mSourceDir;
        final File   mPackageDir;
        final String mPackageName;
        final File   mJavaFile;
        final String mClassName;

        XActionTarget (File sourceDir, File packageDir, String packageName, File javaFile, String className)
        {
            mSourceDir = sourceDir;
            mPackageDir = packageDir;
            mPackageName = packageName;
            mJavaFile = javaFile;
            mClassName = className;
        }
        
    }

    abstract class BAction extends AbstractAction implements Runnable
    {
        ArrayList<XReentrantLock> mLocks;
        String mType;
        XActionTarget mTarget;
        

        BAction(String type, String name, XActionTarget target)
        {
            super();
            mLocks = new ArrayList<XReentrantLock>();
            mType = type;
            putValue(AbstractAction.NAME, name);
            mTarget = target;
        }

        void addLock(XReentrantLock lock)
        {
            mLocks.add(lock);
        }

        @Override
        public void actionPerformed(ActionEvent e)
        {
            if (LOGGING) LOGFILE.println("BAction.actionPerformed(...)");
            startExecutionThread();
        }

        public void startExecutionThread()
        {
            (new Thread(this, mType)).start();
        }

        public void run()
        {
            aquireLocksAndExecute(0);
        }

        void aquireLocksAndExecute(int n)
        {
            if (mLocks.size() <= n)
            {
                // no (further) lock to aquire
                execute();
                return;
            }

            XReentrantLock lock = mLocks.get(n);
            String threadname;
            while (null == (threadname = lock.getOwnerName()))
            {
                if (!lock.tryLock()) continue;

                try
                {
                    // aquire next lock
                    aquireLocksAndExecute(n+1);
                }
                finally
                {
                    lock.unlock();
                }

                return;
            }

            if (threadname.equals(mType))
            {
                if (threadname.equals(NXT_BROWSER_ACTION))
                {
                    showMessageDialog(MSG_NXT_BROWSER_STARTED);
                    return;
                }
                if (threadname.equals(NXT_CONSOLE_ACTION))
                {
                    showMessageDialog(MSG_NXT_CONSOLE_STARTED);
                    return;
                }
                if (threadname.equals(NXT_FLASH_ACTION))
                {
                    showMessageDialog(MSG_NXT_FLASHTOOL_STARTED);
                    return;
                }
                if (threadname.equals(NXT_COMPILE_ACTION) || threadname.equals(NXT_COMPILEPACKAGE_ACTION))
                {
                    showMessageDialog(MSG_COMPILATION_RUNNING);
                    return;
                }
                if (threadname.equals(NXT_UPLOAD_ACTION))
                {
                    showMessageDialog(MSG_NXT_UPLOAD_RUNNING);
                    return;
                }
                if (threadname.equals(EV3_CONTROL_ACTION))
                {
                    showMessageDialog(MSG_EV3_CONTROL_STARTED);
                    return;
                }
                if (threadname.equals(EV3_SDCARD_ACTION))
                {
                    showMessageDialog(MSG_EV3_SDCARD_STARTED);
                    return;
                }
                if (threadname.equals(EV3_LIBS_UPLOAD_ACTION))
                {
                    showMessageDialog(MSG_EV3_LIBS_RUNNING);
                    return;
                }
                if (threadname.equals(EV3_COMPILE_ACTION) || threadname.equals(EV3_COMPILEPACKAGE_ACTION))
                {
                    showMessageDialog(MSG_COMPILATION_RUNNING);
                    return;
                }
                if (threadname.equals(EV3_UPLOAD_ACTION))
                {
                    showMessageDialog(MSG_EV3_UPLOAD_RUNNING);
                    return;
                }
                if (threadname.equals(SIMULATE_ACTION))
                {
                    showMessageDialog(MSG_SIM_STARTED);
                    return;
                }
            }
            else
            {
                if (threadname.equals(NXT_BROWSER_ACTION))
                {
                    showMessageDialog(MSG_NXT_BROWSER_RUNNING);
                    return;
                }
                if (threadname.equals(NXT_CONSOLE_ACTION))
                {
                    showMessageDialog(MSG_NXT_CONSOLE_RUNNING);
                    return;
                }
                if (threadname.equals(NXT_FLASH_ACTION))
                {
                    showMessageDialog(MSG_NXT_FLASHTOOL_RUNNING);
                    return;
                }
                if (threadname.equals(NXT_COMPILE_ACTION) || threadname.equals(NXT_COMPILEPACKAGE_ACTION))
                {
                    showMessageDialog(MSG_COMPILATION_RUNNING);
                    return;
                }
                if (threadname.equals(NXT_UPLOAD_ACTION))
                {
                    showMessageDialog(MSG_NXT_UPLOAD_RUNNING);
                    return;
                }
                if (threadname.equals(EV3_CONTROL_ACTION))
                {
                    showMessageDialog(MSG_EV3_CONTROL_RUNNING);
                    return;
                }
                if (threadname.equals(EV3_SDCARD_ACTION))
                {
                    showMessageDialog(MSG_EV3_SDCARD_RUNNING);
                    return;
                }
                if (threadname.equals(EV3_LIBS_UPLOAD_ACTION))
                {
                    showMessageDialog(MSG_EV3_LIBS_RUNNING);
                    return;
                }
                if (threadname.equals(EV3_COMPILE_ACTION) || threadname.equals(EV3_COMPILEPACKAGE_ACTION))
                {
                    showMessageDialog(MSG_COMPILATION_RUNNING);
                    return;
                }
                if (threadname.equals(EV3_UPLOAD_ACTION))
                {
                    showMessageDialog(MSG_EV3_UPLOAD_RUNNING);
                    return;
                }
                if (threadname.equals(SIMULATE_ACTION))
                {
                    showMessageDialog(MSG_SIM_RUNNING);
                    return;
                }
            }
        }

        abstract protected void execute();

        protected Process startNXTBrowserProcess() throws IOException
        {
            if (LOGGING) LOGFILE.println("BAction.startNXTBrowserProcess()");
            synchronized (mPB)
            {
                mCommandList.clear();
                mCommandList.add(mNXTJava);
                mCommandList.add("-Dnxj.home=" + mNXTHome);
                mCommandList.add("-DCOMMAND_NAME=nxjbrowse");
                mCommandList.add("-classpath");
                mCommandList.add(mNXTClasspathPC);
                mCommandList.add("lejos.pc.tools.NXJBrowser");
                return mPB.start();
            }
        }

        protected Process startNXTConsoleProcess() throws IOException
        {
            if (LOGGING) LOGFILE.println("BAction.startNXTConsoleProcess()");
            synchronized (mPB)
            {
                mCommandList.clear();
                mCommandList.add(mNXTJava);
                mCommandList.add("-Dnxj.home=" + mNXTHome);
                mCommandList.add("-DCOMMAND_NAME=nxjconsoleviewer");
                mCommandList.add("-classpath");
                mCommandList.add(mNXTClasspathPC);
                mCommandList.add("lejos.pc.tools.NXJConsoleViewer");
                return mPB.start();
            }
        }

        protected Process startNXTFlashProcess() throws IOException
        {
            if (LOGGING) LOGFILE.println("BAction.startNXTFlashProcess()");
            synchronized (mPB)
            {
                mCommandList.clear();
                mCommandList.add(mNXTJava);
                mCommandList.add("-Dnxj.home=" + mNXTHome);
                mCommandList.add("-DCOMMAND_NAME=nxjflashg");
                mCommandList.add("-classpath");
                mCommandList.add(mNXTClasspathPC);
                mCommandList.add("lejos.pc.tools.NXJFlashG");
                return mPB.start();
            }
        }

        private void prepareNXTCompileCommand()
        {
            mCommandList.clear();
            mCommandList.add(mNXTJavac);
            mCommandList.add("-source");
            mCommandList.add(JAVA_VERSION_NXT);
            mCommandList.add("-target");
            mCommandList.add(JAVA_VERSION_NXT);
            mCommandList.add("-bootclasspath");
            mCommandList.add(mNXTRuntime);
            if (null != mNXTClasspath)
            {
                mCommandList.add("-classpath");
                mCommandList.add(mNXTClasspath);
            }
            //mCommandList.add("-implicit:class");
            mCommandList.add("-sourcepath");
        }

       protected Process startNXTCompilePackageProcess() throws IOException
       {
           if (LOGGING) LOGFILE.println("BAction.startNXTCompilePackageProcess()");
           synchronized (mPB)
           {
               prepareNXTCompileCommand();
               mCommandList.add(decoratedPath(mTarget.mSourceDir.getAbsolutePath()));  // SOURCEPATH
               File[] files = mTarget.mPackageDir.listFiles();  // look for java files in package directory
               for (File item : files)
               {
                   if (item.isFile())
                   {
                       String lowerCaseName = item.getName().toLowerCase();
                       if (lowerCaseName.endsWith(".java"))
                       {
                           mCommandList.add(decoratedPath(item.getAbsolutePath()));  // JAVA
                       }
                       else if ((lowerCaseName.endsWith(".ctxt")) || (lowerCaseName.endsWith(".class")))
                       {
                           if (item.canWrite()) item.delete();   // delete binaries
                       }
                   }
               }
               return mPB.start();
           }
       }

        protected Process startNXTCompileProcess() throws IOException
        {
            if (LOGGING) LOGFILE.println("BAction.startNXTCompileProcess()");
            synchronized (mPB)
            {
                prepareNXTCompileCommand();
                mCommandList.add(decoratedPath(mTarget.mSourceDir.getAbsolutePath())); // SOURCEPATH
                mCommandList.add(decoratedPath(mTarget.mJavaFile.getAbsolutePath()));  // JAVAFILE
                return mPB.start();
            }
        }

        protected Process startNXTUploadProcess() throws IOException
        {
            if (LOGGING) LOGFILE.println("BAction.startNXTUploadProcess()");
            synchronized (mPB)
            {
                mCommandList.clear();
                mCommandList.add(mNXTJava);
                mCommandList.add("-Dnxj.home=" + mNXTHome);
                mCommandList.add("-DCOMMAND_NAME=nxjlinkandupload");
                mCommandList.add("-classpath");
                mCommandList.add(mNXTClasspathPC);
                mCommandList.add("lejos.pc.tools.NXJLinkAndUpload");
                //mCommandList.add("--usb");
                mCommandList.add("--verbose");
                mCommandList.add("--writeorder");
                mCommandList.add("LE");
                mCommandList.add("--bootclasspath");
                mCommandList.add(mNXTRuntime);
                mCommandList.add("--classpath");
                mCommandList.add((null != mNXTClasspath) ? (mNXTClasspath + File.pathSeparator + decoratedPath(mTarget.mSourceDir.getAbsolutePath())) : decoratedPath(mTarget.mSourceDir.getAbsolutePath())); // CLASSPATH
                mCommandList.add(mTarget.mClassName);
                return mPB.start();
            }
        }

        protected Process startEV3ControlProcess() throws IOException
        {
            if (LOGGING) LOGFILE.println("BAction.startEV3ControlProcess()");
            synchronized (mPB)
            {
                mCommandList.clear();
                mCommandList.add(mEV3Java);
                mCommandList.add("-Dev3.home=" + mEV3Home);
                mCommandList.add("-DCOMMAND_NAME=ev3control");
                mCommandList.add("-classpath");
                mCommandList.add(mEV3ClasspathPC);
                mCommandList.add("lejos.ev3.tools.EV3Control");
                return mPB.start();
            }
        }

        protected Process startEV3SDCardProcess() throws IOException
        {
            if (LOGGING) LOGFILE.println("BAction.startEV3SDCardProcess()");
            synchronized (mPB)
            {
                mCommandList.clear();
                mCommandList.add(mEV3Java);
                mCommandList.add("-Dev3.home=" + mEV3Home);
                mCommandList.add("-DCOMMAND_NAME=ev3sdcard");
                mCommandList.add("-classpath");
                mCommandList.add(decoratedPath(getJaboticsToolsPath()));
                mCommandList.add("org.jabotics.tools.EV3SDCard");
                return mPB.start();
            }
        }

        protected Process startEV3LibUploadProcess(String robot_id, File file) throws IOException
        {
            if (LOGGING) LOGFILE.println("BAction.startEV3LibUploadProcess()");
            synchronized (mPB)
            {
                mCommandList.clear();
                mCommandList.add(mEV3Java);
                mCommandList.add("-Dev3.home=" + mEV3Home);
                mCommandList.add("-DCOMMAND_NAME=ev3scpupload");
                mCommandList.add("-classpath");
                mCommandList.add(mEV3ClasspathPC);
                mCommandList.add("lejos.ev3.tools.EV3ScpUpload");
                mCommandList.add("-n");
                mCommandList.add(robot_id);
                mCommandList.add(file.getCanonicalPath());
                mCommandList.add("/home/lejos/lib/"+file.getName());
                return mPB.start();
            }
        }

        private void prepareEV3CompileCommand()
        {
            mCommandList.clear();
            mCommandList.add(mEV3Javac);
            mCommandList.add("-verbose");
            mCommandList.add("-source");
            mCommandList.add(JAVA_VERSION_EV3);
            mCommandList.add("-target");
            mCommandList.add(JAVA_VERSION_EV3);
            mCommandList.add("-bootclasspath");
            mCommandList.add(mEV3Runtime);  // Dummy runtime to avoid compiler warning for missing bootclasspath
            mCommandList.add("-classpath");
            mCommandList.add(mEV3Classpath);
            //mCommandList.add("-implicit:class");
            mCommandList.add("-sourcepath");
        }

        protected Process startEV3CompilePackageProcess() throws IOException
        {
            if (LOGGING) LOGFILE.println("BAction.startEV3CompilePackageProcess()");
            synchronized (mPB)
            {
                prepareEV3CompileCommand();
                mCommandList.add(decoratedPath(mTarget.mSourceDir.getAbsolutePath()));  // SOURCEPATH
                File[] files = mTarget.mPackageDir.listFiles();  // look for java files in package directory
                for (File item : files)
                {
                    if (item.isFile())
                    {
                        String lowerCaseName = item.getName().toLowerCase();
                        if (lowerCaseName.endsWith(".java"))
                        {
                            mCommandList.add(decoratedPath(item.getAbsolutePath()));  // JAVA
                        }
                        else if ((lowerCaseName.endsWith(".ctxt")) || (lowerCaseName.endsWith(".class")))
                        {
                            if (item.canWrite()) item.delete();   // delete binaries
                        }
                    }
                }
                return mPB.start();
            }
        }

        protected Process startEV3CompileProcess() throws IOException
        {
            if (LOGGING) LOGFILE.println("BAction.startEV3CompileProcess()");
            synchronized (mPB)
            {
                prepareEV3CompileCommand();
                mCommandList.add(decoratedPath(mTarget.mSourceDir.getAbsolutePath())); // SOURCEPATH
                mCommandList.add(decoratedPath(mTarget.mJavaFile.getAbsolutePath()));  // JAVAFILE
                return mPB.start();
            }
        }

        protected Process startEV3JarProcess() throws IOException
        {
            if (LOGGING) LOGFILE.println("BAction.startEV3JarProcess()");
            synchronized (mPB)
            {
                String jarPathName = mTarget.mJavaFile.getAbsolutePath();
                jarPathName = jarPathName.substring(0,jarPathName.length()-5) + ".jar"; // jar file to upload

                mCommandList.clear();
                mCommandList.add(mEV3Jar);
                //mCommandList.add("-c0mfe"); // uncompressed: does not seem to speed up start procedure
                mCommandList.add("-cmfe");
                mCommandList.add(decoratedPath(mEV3LibUser + File.separatorChar + "manifest-ev3"));
                mCommandList.add(decoratedPath(jarPathName));
                mCommandList.add(mTarget.mClassName); // class to execute
                Iterator<String> it = mClassFileFilter.getClassFileIterator();
                int start_index = mTarget.mSourceDir.getAbsolutePath().length() + 1;
                while (it.hasNext())
                {
                	mCommandList.add(it.next().substring(start_index));
                }

                // save current working directory of process builder
                File wd = mPB.directory();
                // set robot source directory as working directory
                mPB.directory(mTarget.mSourceDir);
                Process process = mPB.start();
                // restore working directory of process builder
                mPB.directory(wd);
                
                return process;
            }
        }

        protected Process startEV3UploadProcess(String robot_id) throws IOException
        {
            if (LOGGING) LOGFILE.println("BAction.startEV3UploadProcess()");
            synchronized (mPB)
            {
                String jarPathName = mTarget.mJavaFile.getAbsolutePath();
                jarPathName = jarPathName.substring(0,jarPathName.length()-5) + ".jar"; // jar file to upload

                mCommandList.clear();
                mCommandList.add(mEV3Java);
                mCommandList.add("-Dev3.home=" + mEV3Home);
                mCommandList.add("-DCOMMAND_NAME=ev3scpupload");
                mCommandList.add("-classpath");
                mCommandList.add(mEV3ClasspathPC);
                mCommandList.add("lejos.ev3.tools.EV3ScpUpload");
                mCommandList.add("-n");
                mCommandList.add(robot_id);
                mCommandList.add(decoratedPath(jarPathName));
                // switch jarPathName to EV3 path name
                jarPathName = "/home/lejos/programs/" + jarPathName.substring(jarPathName.lastIndexOf(File.separatorChar));
                mCommandList.add(decoratedPath(jarPathName));
                return mPB.start();
            }
        }

        protected Process startSimulationProcess() throws IOException
        {
            if (LOGGING) LOGFILE.println("BAction.startSimulationProcess()");

            synchronized (mPB)
            {
                mCommandList.clear();
                mCommandList.add(mSIMJava);
                if (RMI_PORT > 0) mCommandList.add("-D" + SIMDefs.KEY_RMI_PORT + "=" + RMI_PORT);
                mCommandList.add("-classpath");
                // clath path containing jabotics-sim.jar and robot source directory
                mCommandList.add(mSIMClasspath + File.pathSeparator + decoratedPath(mTarget.mSourceDir.getAbsolutePath()));
                mCommandList.add("org.jabotics.robot.sim.XSimulation");
                mCommandList.add("-l");
                mCommandList.add(mLanguage);
                mCommandList.add("-d");
                mCommandList.add(decoratedPath(mTarget.mSourceDir.getAbsolutePath()));
                mCommandList.add(mTarget.mClassName);

                // save current working directory of process builder
                File wd = mPB.directory();
                // set robot source directory as working directory
                mPB.directory(mTarget.mSourceDir);
                Process process = mPB.start();
                // restore working directory of process builder
                mPB.directory(wd);
                
                return process;
            }
        }
     }

     class NXTBrowserAction extends BAction
     {
         public NXTBrowserAction(XActionTarget target)
         {
             super(NXT_BROWSER_ACTION,MSG_NXT_BROWSER,target);
             addLock(sConnectionLock);
         }

         public void execute()
         {
             try
             {
                 Process p = startNXTBrowserProcess();
                 XProcessViewer viewer = new XProcessViewer();
                 viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_NXT_BROWSER);
                 viewer.showOnError(true);
                 viewer.automaticTermination(true);
                 viewer.startLogging(p);
                 p.waitFor();
             }
             catch (IOException ioEx) {}
             catch (InterruptedException irEx) {}
         }
     }

     class NXTConsoleAction extends BAction
     {
         public NXTConsoleAction(XActionTarget target)
         {
             super(NXT_CONSOLE_ACTION,MSG_NXT_CONSOLE,target);
             addLock(sConnectionLock);
         }

         public void execute()
         {
             try
             {
                 Process p = startNXTConsoleProcess();
                 XProcessViewer viewer = new XProcessViewer();
                 viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_NXT_CONSOLE);
                 viewer.showOnError(true);
                 viewer.automaticTermination(true);
                 viewer.startLogging(p);
                 p.waitFor();
             }
             catch (IOException ioEx) {}
             catch (InterruptedException irEx) {}
         }
     }

     class NXTFlashAction extends BAction
     {
         public NXTFlashAction(XActionTarget target)
         {
             super(NXT_FLASH_ACTION,MSG_NXT_FLASHTOOL,target);
             addLock(sConnectionLock);
         }

         public void execute()
         {
             try
             {
                 Process p = startNXTFlashProcess();
                 XProcessViewer viewer = new XProcessViewer();
                 viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_NXT_FLASHTOOL);
                 viewer.showOnError(true);
                 viewer.automaticTermination(true);
                 viewer.startLogging(p);
                 p.waitFor();
             }
             catch (IOException ioEx) {}
             catch (InterruptedException irEx) {}
         }
     }

     class NXTCompilePackageAction extends BAction
     {
         public NXTCompilePackageAction(XActionTarget target)
         {
             super(NXT_COMPILEPACKAGE_ACTION,MSG_NXT_COMPILEPACKAGE,target);
             addLock(sCompileLock);
         }

         public void execute()
         {
             try
             {
                 Process p = startNXTCompilePackageProcess();
                 XProcessViewer viewer = new XProcessViewer();
                 viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_NXT_COMPILEPACKAGE);
                 viewer.getOutErrStreamPanel().getOutTA().append(MSG_COMPILING_PACKAGE + " " + mTarget.mPackageDir.getName() + " ...\n");
                 viewer.setVisible(true);
                 viewer.startLogging(p);
                 if (!viewer.waitFor())
                 {
                     // Viewer vorzeitig beendet
                     p.destroy();
                     viewer.dispose();
                     return;
                 }
                 viewer.getOutErrStreamPanel().getOutTA().append(((0 == viewer.GetProcessExitValue()) ? MSG_COMPILATION_COMPLETED : MSG_COMPILATION_FAILED) + "\n");
                 viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
             }
             catch (IOException ioEx) {}
             //catch (InterruptedException irEx) {}
         }
     }

     class NXTCompileAction extends BAction
     {
         public NXTCompileAction(XActionTarget target)
         {
             super(NXT_COMPILE_ACTION,MSG_NXT_COMPILE,target);
             addLock(sCompileLock);
         }

         public void execute()
         {
             try
             {
                 Process p = startNXTCompileProcess();
                 XProcessViewer viewer = new XProcessViewer();
                 viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_NXT_COMPILE);
                 viewer.getOutErrStreamPanel().getOutTA().append(MSG_COMPILING + " " + mTarget.mClassName + " ...\n");
                 viewer.setVisible(true);
                 viewer.startLogging(p);
                 if (!viewer.waitFor())
                 {
                     // Viewer vorzeitig beendet
                     p.destroy();
                     viewer.dispose();
                     return;
                 }
                 viewer.getOutErrStreamPanel().getOutTA().append(((0 == viewer.GetProcessExitValue()) ? MSG_COMPILATION_COMPLETED : MSG_COMPILATION_FAILED) + "\n");
                 viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
             }
             catch (IOException ioEx) {}
             //catch (InterruptedException irEx) {}
         }
     } 


     class NXTUploadAction extends BAction
     {
         public NXTUploadAction(XActionTarget target)
         {
             super(NXT_UPLOAD_ACTION,MSG_NXT_UPLOAD,target);
             addLock(sConnectionLock);
             addLock(sCompileLock);
         }

         public void execute()
         {
             try
             {
                 Process p = startNXTCompileProcess();
                 XProcessViewer viewer = new XProcessViewer();
                 viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_NXT_UPLOAD);
                 viewer.getOutErrStreamPanel().getOutTA().append(MSG_COMPILING + " " + mTarget.mClassName + " ...\n");
                 viewer.setVisible(true);
                 viewer.startLogging(p);
                 if (!viewer.waitFor())
                 {
                     // Viewer vorzeitig beendet
                     p.destroy();
                     viewer.dispose();
                     return;
                 }
                 if (0 == viewer.GetProcessExitValue())
                 {
                     viewer.getOutErrStreamPanel().getOutTA().append(MSG_COMPILATION_COMPLETED + "\n\n");
                     viewer.getOutErrStreamPanel().getOutTA().append(MSG_NXT_UPLOAD + " ...\n\n");
                 }
                 else
                 {
                     viewer.getOutErrStreamPanel().getOutTA().append(MSG_COMPILATION_FAILED + "\n");
                     viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
                     return;
                 }
                 p = startNXTUploadProcess();
                 viewer.startLogging(p);
                 if (!viewer.waitFor())
                 {
                     // Viewer vorzeitig beendet
                     p.destroy();
                     viewer.dispose();
                     return;
                 }
                 viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
                 //p.waitFor();
             }
             catch (IOException ioEx) {}
             //catch (InterruptedException irEx) {}
         }
     }

    class EV3SelectRobotAction extends BAction
    {
        public EV3SelectRobotAction(XActionTarget target)
        {
            super(EV3_SELECT_ROBOT_ACTION,MSG_EV3_ROBOT,target);
            addLock(sConnectionLock);
        }

        public void execute()
        {
            String robot_id = showInputDialog(MSG_EV3_ROBOT_ADDRESS,getPropertyString("org.jabotics.robot.id","10.0.1.1"));
            if ( (null != robot_id) && (robot_id.length() > 0) )
            {
                setPropertyString("org.jabotics.robot.id",robot_id);
            }
        }
    }

    class EV3ControlAction extends BAction
    {
        public EV3ControlAction(XActionTarget target)
        {
            super(EV3_CONTROL_ACTION,MSG_EV3_CONTROL,target);
            addLock(sConnectionLock);
        }

        public void execute()
        {
            try
            {
                Process p = startEV3ControlProcess();
                XProcessViewer viewer = new XProcessViewer();
                viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_EV3_CONTROL);
                viewer.showOnError(true);
                viewer.automaticTermination(true);
                viewer.startLogging(p);
                p.waitFor();
            }
            catch (IOException ioEx) {}
            catch (InterruptedException irEx) {}
        }
    }

    class EV3SDCardAction extends BAction
    {
        public EV3SDCardAction(XActionTarget target)
        {
            super(EV3_SDCARD_ACTION,MSG_EV3_SDCARD,target);
            addLock(sConnectionLock);
        }

        public void execute()
        {
            try
            {
                Process p = startEV3SDCardProcess();
                XProcessViewer viewer = new XProcessViewer();
                viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_EV3_SDCARD);
                viewer.showOnError(true);
                viewer.automaticTermination(true);
                viewer.startLogging(p);
                p.waitFor();
            }
            catch (IOException ioEx) {}
            catch (InterruptedException irEx) {}
        }
    }


    class EV3LibsUploadAction extends BAction
    {
        public EV3LibsUploadAction(XActionTarget target)
        {
            super(EV3_LIBS_UPLOAD_ACTION,MSG_EV3_LIBS,target);
            addLock(sConnectionLock);
        }

        public void execute()
        {
            if (mEV3UserlibFiles.isEmpty())
            {
                System.err.println(MSG_EV3_LIBS_FAILED);
                return;
            }

            String robot_id = getPropertyString("org.jabotics.robot.id","10.0.1.1");

            XProcessViewer viewer = new XProcessViewer();
            viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_EV3_LIBS);
            viewer.getOutErrStreamPanel().getOutTA().append(MSG_EV3_LIBS + " -> " + robot_id + " ...\n");
            viewer.setVisible(true);
            for (File file : mEV3UserlibFiles)
            {
                try
                {
                    viewer.getOutErrStreamPanel().getOutTA().append(file.getName() + " ... ");
                    Process p = startEV3LibUploadProcess(robot_id,file);
                    viewer.startLogging(p);
                    if (!viewer.waitFor())
                    {
                        // Viewer vorzeitig beendet
                        p.destroy();
                        viewer.dispose();
                        return;
                    }
                    viewer.getOutErrStreamPanel().getOutTA().append((0 == viewer.GetProcessExitValue()) ? MSG_EV3_UPLOAD_COMPLETED + "\n" : MSG_EV3_UPLOAD_FAILED + "\n");
                }
                catch (IOException ioEx) {}
            }
            viewer.getOutErrStreamPanel().getOutTA().append(MSG_EV3_LIBS_COMPLETED + "\n");
            viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        }
    }

    class EV3CompilePackageAction extends BAction
    {
        public EV3CompilePackageAction(XActionTarget target)
        {
            super(EV3_COMPILEPACKAGE_ACTION,MSG_EV3_COMPILEPACKAGE,target);
            addLock(sCompileLock);
        }

        public void execute()
        {
            try
            {
                Process p = startEV3CompilePackageProcess();
                XProcessViewer viewer = new XProcessViewer();
                viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_EV3_COMPILEPACKAGE);
                viewer.getOutErrStreamPanel().getOutTA().append(MSG_COMPILING_PACKAGE + " " + mTarget.mPackageDir.getName() + " ...\n");
                viewer.setVisible(true);
                viewer.startLogging(p);
                if (!viewer.waitFor())
                {
                    // Viewer vorzeitig beendet
                    p.destroy();
                    viewer.dispose();
                    return;
                }
                viewer.getOutErrStreamPanel().getOutTA().append(((0 == viewer.GetProcessExitValue()) ? MSG_COMPILATION_COMPLETED : MSG_COMPILATION_FAILED) + "\n");
                viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
            }
            catch (IOException ioEx) {}
            //catch (InterruptedException irEx) {}
        }
    }

    class EV3CompileAction extends BAction
    {
        public EV3CompileAction(XActionTarget target)
        {
            super(EV3_COMPILE_ACTION,MSG_EV3_COMPILE,target);
            addLock(sCompileLock);
        }

        public void execute()
        {
            try
            {
                Process p = startEV3CompileProcess();
                XProcessViewer viewer = new XProcessViewer();
                viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_EV3_COMPILE);
                viewer.getOutErrStreamPanel().getOutTA().append(MSG_COMPILING + " " + mTarget.mClassName + " ...\n");
                viewer.setVisible(true);
                mClassFileFilter.init();
                viewer.getOutErrStreamPanel().getErrTA().setFilter(mClassFileFilter);
                viewer.startLogging(p);
                if (!viewer.waitFor())
                {
                    // Viewer vorzeitig beendet
                    p.destroy();
                    viewer.dispose();
                    return;
                }
                if (0 == viewer.GetProcessExitValue())
                {
                    viewer.getOutErrStreamPanel().getOutTA().append(MSG_COMPILATION_COMPLETED + "\n\n");
                    viewer.getOutErrStreamPanel().getOutTA().append(MSG_JAR + " ...\n");
                }
                else
                {
                    viewer.getOutErrStreamPanel().getOutTA().append(MSG_COMPILATION_FAILED + "\n");
                    viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
                    return;
                }
                p = startEV3JarProcess();
                viewer.startLogging(p);
                if (!viewer.waitFor())
                {
                    // Viewer vorzeitig beendet
                    p.destroy();
                    viewer.dispose();
                    return;
                }
                viewer.getOutErrStreamPanel().getOutTA().append((0 == viewer.GetProcessExitValue()) ? MSG_JAR_COMPLETED + "\n" : MSG_JAR_FAILED + "\n");
                viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
            }
            catch (IOException ioEx) {}
            //catch (InterruptedException irEx) {}
        }
    }

    class EV3UploadAction extends BAction
    {
        public EV3UploadAction(XActionTarget target)
        {
            super(EV3_UPLOAD_ACTION,MSG_EV3_UPLOAD,target);
            addLock(sConnectionLock);
            addLock(sCompileLock);
        }

        public void execute()
        {
            try
            {
                String robot_id = getPropertyString("org.jabotics.robot.id","10.0.1.1");

                Process p = startEV3CompileProcess();
                XProcessViewer viewer = new XProcessViewer();
                viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_EV3_UPLOAD);
                viewer.getOutErrStreamPanel().getOutTA().append(MSG_COMPILING + " " + mTarget.mClassName + " ...\n");
                viewer.setVisible(true);
                mClassFileFilter.init();
                viewer.getOutErrStreamPanel().getErrTA().setFilter(mClassFileFilter);
                viewer.startLogging(p);
                if (!viewer.waitFor())
                {
                    // Viewer vorzeitig beendet
                    p.destroy();
                    viewer.dispose();
                    return;
                }
                if (0 == viewer.GetProcessExitValue())
                {
                    viewer.getOutErrStreamPanel().getOutTA().append(MSG_COMPILATION_COMPLETED + "\n\n");
                    viewer.getOutErrStreamPanel().getOutTA().append(MSG_JAR + " ...\n");
                }
                else
                {
                    viewer.getOutErrStreamPanel().getOutTA().append(MSG_COMPILATION_FAILED + "\n");
                    viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
                    return;
                }
                p = startEV3JarProcess();
                viewer.startLogging(p);
                if (!viewer.waitFor())
                {
                    // Viewer vorzeitig beendet
                    p.destroy();
                    viewer.dispose();
                    return;
                }
                if (0 == viewer.GetProcessExitValue())
                {
                    viewer.getOutErrStreamPanel().getOutTA().append(MSG_JAR_COMPLETED + "\n\n");
                    viewer.getOutErrStreamPanel().getOutTA().append(MSG_EV3_UPLOAD + " -> " + robot_id + " ...\n");
                }
                else
                {
                    viewer.getOutErrStreamPanel().getOutTA().append(MSG_JAR_FAILED + "\n");
                    viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
                    return;
                }
                p = startEV3UploadProcess(robot_id);
                viewer.startLogging(p);
                if (!viewer.waitFor())
                {
                    // Viewer vorzeitig beendet
                    p.destroy();
                    viewer.dispose();
                    return;
                }
                viewer.getOutErrStreamPanel().getOutTA().append((0 == viewer.GetProcessExitValue()) ? MSG_EV3_UPLOAD_COMPLETED + "\n" : MSG_EV3_UPLOAD_FAILED + "\n");
                viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
                //p.waitFor();
            }
            catch (IOException ioEx) {}
            //catch (InterruptedException irEx) {}
        }
    }

    class SimulateAction extends BAction
    {
        public SimulateAction(XActionTarget target)
        {
            super(SIMULATE_ACTION,MSG_SIM_START,target);
            addLock(sCompileLock);
        }

        public void execute()
        {
            try
            {
                if (!tryCompileClass(mTarget)) return;

                Process p = startSimulationProcess();
                XProcessViewer viewer = new XProcessViewer();
                viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_SIM_START);
                viewer.showOnError(true);
                viewer.startLogging(p);
                viewer.waitFor();
                if (viewer.isVisible())
                {
                    viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
                }
                else
                {
                    p.destroy();
                    viewer.dispose();
                }
            }
            catch (IOException ioEx) {}
        }
    }


    /*static public void test()
    {
        XBlueJExtension extension = new XBlueJExtension();
        extension.mLanguage = "de";
        extension.initMessages();
    }*/

    /*static public void test()
    {
        try
        {
            String robot_id = mBlueJ.getExtensionPropertyString("org.jabotics.robot.id","10.0.1.1");

            mCommandList.clear();
            mCommandList.addAll(mCommandList);
            mCommandList.add(robot_id);
            mCommandList.add(mJaboticsEV3);
            mCommandList.add("/home/lejos/lib/jabotics-ev3.jar");
            Process p = mPB.start();
            XProcessViewer viewer = new XProcessViewer();
            viewer.setTitle(MSG_ROBOT_EXT + ": " + MSG_EV3_LIBS);
            viewer.getOutErrStreamPanel().getOutTA().append(MSG_EV3_LIBS + " -> " + robot_id + " ...\n");
            viewer.setVisible(true);
            viewer.startLogging(p);
            if (!viewer.waitFor())
            {
                // Viewer vorzeitig beendet
                p.destroy();
                viewer.dispose();
                return;
            }
            viewer.getOutErrStreamPanel().getOutTA().append((0 == viewer.GetProcessExitValue()) ? MSG_EV3_LIBS_COMPLETED + "\n" : MSG_EV3_LIBS_FAILED + "\n");
            viewer.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        }
        catch (IOException ioEx) {}
    }*/

    /*static public void testUbuntu()
    {
        try
        {
            Runtime r = Runtime.getRuntime();
            //Process p = r.exec("java -classpath D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterBlueJErweiterung org.jabotics.nxt.bluej.XTestprozess");
            //Process p = r.exec("java -classpath \"C:\\Programme\\lejos NXJ\\lib\\pc\\pctools.jar\" lejos.pc.tools.NXJFlashG");
            //Process p = r.exec("javac -bootclasspath \"C:\\Programme\\lejos NXJ\\lib\\nxt\\classes.jar\" -extdirs \"C:\\Programme\\BlueJ\\lib\\userlib\" D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\\XRobot2.java");
            //Process p = r.exec("javac -bootclasspath \"C:\\Programme\\lejos NXJ\\lib\\nxt\\classes.jar\" -extdirs \"C:\\Programme\\BlueJ\\lib\\userlib\" -implicit:class -sourcepath \"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\" \"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\\XRobot2.java\"");
            Process p = r.exec("java -Dnxj.home=\"/home/ubuntu/Robotik/leJOS_NXJ_0.9.1beta-3\" -DCOMMAND_NAME=nxjlink -classpath \"/home/ubuntu/Robotik/leJOS_NXJ_0.9.1beta-3/lib/pc/pctools.jar\" lejos.pc.tools.NXJLink --writeorder \"LE\" --bootclasspath \"/home/ubuntu/Robotik/leJOS_NXJ_0.9.1beta-3/lib/nxt/classes.jar\" --extdirs \"/home/ubuntu/Robotik/leJOS_NXJ_0.9.1beta-3/lib/userlib\" --classpath \"/media/Daten/Programmierung/BlueJ/StandardRoboterTest\" --\"/media/Daten/Programmierung/BlueJ/StandardRoboterTest/XRobot.class\"");
            // java -Dnxj.home="/home/ubuntu/Robotik/leJOS_NXJ_0.9.1beta-3" -DCOMMAND_NAME=nxjlink -classpath "/home/ubuntu/Robotik/leJOS_NXJ_0.9.1beta-3/lib/pc/pctools.jar" lejos.pc.tools.NXJLink --writeorder \"LE\" --bootclasspath "/home/ubuntu/Robotik/leJOS_NXJ_0.9.1beta-3/lib/nxt/classes.jar" --extdirs "/home/ubuntu/Robotik/leJOS_NXJ_0.9.1beta-3/lib/userlib" --classpath "/media/Daten/Dokumente/Programmierung/BlueJ/StandardRoboterTest" --"/media/Daten/Dokumente/Programmierung/BlueJ/StandardRoboterTest/XRobot.class"

            XProcessViewer viewer = new XProcessViewer();
            viewer.setTitle("NXJ-Erweiterung");
            viewer.setVisible(true);
            viewer.getOutErrStreamPanel().getOutTA().append("Ausgabe des Prozesses:\n");
            viewer.getErrTA().append("Fehlermeldungen:\n");
            viewer.startLogging(p);
            p.waitFor();
            viewer.stopLogging();
            try
            {
                Thread.sleep(30000);
            }
            catch(InterruptedException e) {}
        }
        catch (IOException ex) {}
        catch (InterruptedException ex) {}
        System.exit(0);
    }*/

    /*public void test2Ubuntu()
    {
        //startup();

        try
        {
            //Process p = r.exec("java -classpath D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterBlueJErweiterung org.jabotics.nxt.bluej.XTestprozess");
            //Process p = r.exec("java -classpath \"C:\\Programme\\lejos NXJ\\lib\\pc\\pctools.jar\" lejos.pc.tools.NXJFlashG");
            //Process p = r.exec("javac -bootclasspath \"C:\\Programme\\lejos NXJ\\lib\\nxt\\classes.jar\" -extdirs \"C:\\Programme\\BlueJ\\lib\\userlib\" D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\\XRobot2.java");
            //Process p = r.exec("javac -bootclasspath \"C:\\Programme\\lejos NXJ\\lib\\nxt\\classes.jar\" -extdirs \"C:\\Programme\\BlueJ\\lib\\userlib\" -implicit:class -sourcepath \"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\" \"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\\XRobot2.java\"");
            mCompileCommandListNXT.set(mCompileCommandListNXT.size()-2, "/media/Daten/Programmierung/BlueJ/StandardRoboterTest");
            mCompileCommandListNXT.set(mCompileCommandListNXT.size()-1, "/media/Daten/Programmierung/BlueJ/StandardRoboterTest/XRobot.java");
            mCommandList.set(mCommandList.size()-2, "/media/Daten/Programmierung/BlueJ/StandardRoboterTest");
            mCommandList.set(mCommandList.size()-1, "XRobot");
            XProcessViewer viewer = new XProcessViewer();
            viewer.setTitle("NXJ-Erweiterung");
            viewer.setVisible(true);
            viewer.getOutErrStreamPanel().getOutTA().append("Ausgabe des Prozesses:\n");
            viewer.getErrTA().append("Fehlermeldungen:\n");
            Process p = mPBCompile.start();
            viewer.startLogging(p);
            p.waitFor();
            viewer.stopLogging();
            p = mPBLinkAndUpload.start();
            viewer.startLogging(p);
            p.waitFor();
            viewer.stopLogging();
            Thread.sleep(30000);
            System.exit(0);
        }
        catch (IOException ioEx) {}
        catch(InterruptedException irEx) {}
    }*/

    /*static public void testWin()
    {
        try
        {
            Runtime r = Runtime.getRuntime();
            //Process p = r.exec("java -classpath D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterBlueJErweiterung org.jabotics.nxt.bluej.XTestprozess");
            //Process p = r.exec("java -classpath \"C:\\Programme\\lejos NXJ\\lib\\pc\\pctools.jar\" lejos.pc.tools.NXJFlashG");
            //Process p = r.exec("javac -bootclasspath \"C:\\Programme\\lejos NXJ\\lib\\nxt\\classes.jar\" -extdirs \"C:\\Programme\\BlueJ\\lib\\userlib\" D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\\XRobot2.java");
            //Process p = r.exec("javac -bootclasspath \"C:\\Programme\\lejos NXJ\\lib\\nxt\\classes.jar\" -extdirs \"C:\\Programme\\BlueJ\\lib\\userlib\" -implicit:class -sourcepath \"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\" \"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\\XRobot2.java\"");
            Process p = r.exec("java -Dnxj.home=\"C:\\Programme\\lejos NXJ\" -DCOMMAND_NAME=nxjlink -classpath \"C:\\Programme\\lejos NXJ\\lib\\pc\\pctools.jar\" lejos.pc.tools.NXJLink --writeorder \"LE\" --bootclasspath \"C:\\Programme\\lejos NXJ\\lib\\nxt\\classes.jar\" --extdirs \"C:\\Programme\\BlueJ\\lib\\userlib\" --classpath \"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\" --\"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\\XRobot.class\"");
            XProcessViewer viewer = new XProcessViewer();
            viewer.setTitle("NXJ-Erweiterung");
            viewer.setVisible(true);
            .append("Ausgabe des Prozesses:\n");
            viewer.getErrTA().append("Fehlermeldungen:\n");
            viewer.startLogging(p);
            p.waitFor();
            viewer.stopLogging();
            try
            {
                Thread.sleep(30000);
            }
            catch(InterruptedException e) {}
        }
        catch (IOException ex) {}
        catch (InterruptedException ex) {}
        System.exit(0);
    }*/

    /*public void test2Win()
    {
        startup();

        try
        {
            //Process p = r.exec("java -classpath D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterBlueJErweiterung org.jabotics.nxt.bluej.XTestprozess");
            //Process p = r.exec("java -classpath \"C:\\Programme\\lejos NXJ\\lib\\pc\\pctools.jar\" lejos.pc.tools.NXJFlashG");
            //Process p = r.exec("javac -bootclasspath \"C:\\Programme\\lejos NXJ\\lib\\nxt\\classes.jar\" -extdirs \"C:\\Programme\\BlueJ\\lib\\userlib\" D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\\XRobot2.java");
            //Process p = r.exec("javac -bootclasspath \"C:\\Programme\\lejos NXJ\\lib\\nxt\\classes.jar\" -extdirs \"C:\\Programme\\BlueJ\\lib\\userlib\" -implicit:class -sourcepath \"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\" \"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\\XRobot2.java\"");
            /*mCompileCommandListNXT.set(mCompileCommandListNXT.size()-2, "\"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\"");
            mCompileCommandListNXT.set(mCompileCommandListNXT.size()-1, "\"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\\XRobot.java\"");
            mCommandList.set(mCommandList.size()-2, "\"D:\\Dokumente\\Programmierung\\BlueJ\\StandardRoboterTest\"");
            mCommandList.set(mCommandList.size()-1, "\"XRobot\"");
            XProcessViewer viewer = new XProcessViewer();
            viewer.setTitle("NXJ-Erweiterung");
            viewer.setVisible(true);
            viewer.getOutTA().append("Ausgabe des Prozesses:\n");
            viewer.getErrTA().append("Fehlermeldungen:\n");
            Process p = mPBCompile.start();
            viewer.startLogging(p);
            p.waitFor();
            viewer.stopLogging();
            p = mPBLinkAndUpload.start();
            viewer.startLogging(p);
            p.waitFor();
            viewer.stopLogging();
            Thread.sleep(30000);
            System.exit(0);
        }
        catch (IOException ioEx) {}
        //catch(InterruptedException irEx) {}
    }*/
}
